<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

function sql_types() {
  if (SETUP_DB_TYPE=="mysql" or SETUP_DB_TYPE=="pgsql") {
    return array(
      "string"=>array("varchar(%s)","255"),
      "text"=>array("text",""), // = 64k
      "int"=>array("decimal(%s) default 0","10"),
      "date"=>array("decimal(%s)","10"),
      "bool"=>array("decimal(%s) default 0","1"),
      "id"=>array("decimal(%s)","15"),
      "float"=>array("float","")
    );
  } else {
    return array(
      "string"=>array("VARCHAR2(%s)","255"),
      "text"=>array("CLOB",""),
      "int"=>array("NUMBER(%s)","10"),
      "date"=>array("NUMBER(%s)","10"),
      "bool"=>array("NUMBER(%s)","1"),
      "id"=>array("NUMBER(%s)","15"),
      "float"=>array("FLOAT","")
    );
  }
}

function sql_table_get_indexes($table) {
  $indexes = array();
  if (SETUP_DB_TYPE=="mysql") {
    $sql = sprintf("show index from %s",$table);
    if (($dbindexes = sql_fetch($sql)) === false) return false;
    foreach ($dbindexes as $index) {
	  $type = "index";
	  if ($index["Key_name"] == "PRIMARY") $type = "primary";
	  if (!isset($indexes[$index["Column_name"]])) $indexes[$index["Column_name"]] = array("primary"=>false,"index"=>false);
	  $indexes[$index["Column_name"]][$type] = true;
    }
  } else if (SETUP_DB_TYPE=="pgsql") {
    $sql = sprintf("select indexname,indexdef from pg_indexes where tablename='%s'",$table);
    if (($dbindexes = sql_fetch($sql)) === false) return false;
    foreach ($dbindexes as $index) {
	  $type = "index";
	  if (strpos($index["indexdef"],"UNIQUE")) $type = "primary";
	  preg_match("/\((.*?)\)/",$index["indexdef"],$match);
	  $keys = explode(",",str_replace("\"","",$match[1]));
	  foreach ($keys as $key) {
	    if (!isset($indexes[$key])) $indexes[$key] = array("primary"=>false,"index"=>false);
	    $indexes[$key][$type] = true;
	  }
    }
  } else {
    $sql = sprintf("select a.column_name,b.uniqueness,b.index_name from user_ind_columns a, user_indexes b where a.index_name=b.index_name and a.table_name=b.table_name and a.table_name='%s'",strtoupper($table));
    if (($dbindexes = sql_fetch($sql)) === false) return false;
    foreach ($dbindexes as $index) {
	  $key = strtolower($index["column_name"]);
	  if (!isset($indexes[$key])) $indexes[$key] = array("primary"=>false,"index"=>false);
	  if ($index["uniqueness"] == "UNIQUE") {
	    $indexes[$key]["primary"] = true;
	  } else {
	    $indexes[$key]["index"] = true;
	  }
    }
  }
  return $indexes;
}

function sql_table_modify_primary($schema_name,$primaries) {
  if (sql_query(sprintf("ALTER TABLE %s ADD PRIMARY KEY (%s)",$schema_name,$primaries))) return true;
  return false;
}

function sql_drop_index($table, $type) {
  if (SETUP_DB_TYPE=="mysql") {
    if (sql_query(sprintf("ALTER TABLE %s DROP %s",$table,$type))) return true;
  } else if (SETUP_DB_TYPE=="pgsql") {
    if ($type=="PRIMARY KEY") {
      if (sql_query(sprintf("ALTER TABLE %s DROP CONSTRAINT %s",$table,$table."_pkey"))) return true;
	} else {
	  if (sql_query(sprintf("DROP INDEX %s",$type))) return true;
	}
  } else {
    if (sql_query(sprintf("DROP INDEX %s",$type))) return true;
  }
  return false;
}

function sql_add_index($type, $name, $table, $field) {
  if (SETUP_DB_TYPE=="mysql") {
    if (sql_query(sprintf("CREATE %s INDEX %s ON %s(%s)",$type,$name,$table,$field))) return true;
  } else if (SETUP_DB_TYPE=="pgsql") {
    // TODO2 tsearch2 for pgsql ??
    if (sql_query(sprintf("CREATE INDEX %s ON %s(%s)",$name,$table,$field))) return true;
  } else {
    // indextype is ctxsys.context, but not for "like"
    if ($type=="FULLTEXT") return true;
	if (sql_query(sprintf("CREATE INDEX %s ON %s(%s)%s",$name,$table,$field,$type))) return true;
  }
  return false;
}

function sql_table_rename($table,$new_table) {
  if (sql_table_exists($new_table)) return true;
  if (!sql_table_exists($table)) return true;
  if (sql_query(sprintf("alter table %s rename to %s",$table,$new_table))) return true;
  return false;
}

function sql_column_rename($table,$column,$new_column) {
  if (sql_column_exists($table,$new_column)) return true;
  if (!sql_column_exists($table,$column)) return true;
  if (SETUP_DB_TYPE=="mysql") {
    $type = "";
    $sql = sprintf("show columns from %s",$table);
    if (($dbcolumns = sql_fetch($sql)) === false) return false;
    foreach ($dbcolumns as $col) {
      if ($col["Field"]==$column) $type = str_replace(",0","",$col["Type"]);
    }
    if (sql_query(sprintf("alter table %s change %s %s %s",$table,$column,$new_column,$type))) return true;
  } else { // pgsql, oracle
    if (sql_query(sprintf("alter table %s rename column %s to %s",$table,$column,$new_column))) return true;
  }
  return false;
}

function sql_table_modify($table,$field_name,$ftype) {
  $type = "";
  if (SETUP_DB_TYPE=="mysql") {
    $sql = sprintf("show columns from %s",$table);
    if (($dbcolumns = sql_fetch($sql)) === false) return false;
    foreach ($dbcolumns as $column) {
      if ($column["Field"]==$field_name) $type = str_replace(",0","",$column["Type"]);
    }
    if ($type!=str_replace(" default 0","",$ftype)) {
      if (!sql_query(sprintf("ALTER TABLE %s MODIFY %s %s",$table,$field_name,$ftype))) return false;
    }
  } else if (SETUP_DB_TYPE=="pgsql") {
    if (($dbcolumns=pg_meta_data($GLOBALS["DB"],$table)) === false) return false;
    foreach ($dbcolumns as $key=>$column) {
      if ($key==$field_name) $type = $column["type"];
    }
	if (strpos($type,"default 0")) $addon = sprintf(", ALTER %s set default 0",$field_name); else $addon = "";
	$ftype = str_replace(" default 0","",$ftype);
    if ($type!=$ftype) {
      if (!sql_query(sprintf("ALTER TABLE %s ALTER %s TYPE %s",$table,$field_name,$ftype).$addon)) return false;
    }
  } else {
    $sql = sprintf("select data_type,data_precision,data_length from user_tab_columns where table_name='%s' and column_name='%s'",strtoupper($table),strtoupper($field_name));
    if (($dbcolumn = sql_fetch($sql,true)) === false) return false;
	if (!empty($dbcolumn["data_type"])) {
      $type = $dbcolumn["data_type"];
	  if (strpos($ftype,"(")) {
	    if ($dbcolumn["data_precision"]!="") $type .= "(".$dbcolumn["data_precision"].")";
	      else if ($dbcolumn["data_length"]!="") $type .= "(".$dbcolumn["data_length"].")";
	  }
      if ($type!=str_replace(" default 0","",$ftype)) {
        if (!sql_query(sprintf("ALTER TABLE %s MODIFY %s %s",$table,$field_name,$ftype))) return false;
	  }
    }
  }
  return true;
}

function sql_table_add($schema_name,$field_name,$ftype) {
  if (sql_query(sprintf("ALTER TABLE %s ADD %s %s",$schema_name,$field_name,$ftype))) return true;
  return false;
}

function sql_table_exists($table) {
  if (sql_query(sprintf("SELECT 1 FROM %s",$table))) return true;
  return false;
}
  
function sql_column_exists($table,$column) {
  if (sql_query(sprintf("SELECT %s FROM %s",$column,$table))) return true;
  return false;
}

function sql_create_database($database) {
  if (SETUP_DB_TYPE=="mysql") {
    return @mysql_query(sprintf("create database %s",$database),$GLOBALS["DB"]);
  } else if (SETUP_DB_TYPE=="pgsql") {
    return @pg_query($GLOBALS["DB"],sprintf("create database %s encoding='UTF8'",$database));
  } else {
    return true;
  }
}

function _xml_get_default_schema_nosql($schema_content,$schema,$is_sys) {
  $result = "";
  if (strpos($schema_content,'<field name="seen"')) {
    $result .= '<rowfilter name="filter_sys1" views="display" simple_type="_fgstyle" function="buildseenstyle|seen"/>';
  }
  if (!strpos($schema_content,'<field name="created"')) $result .= '<field name="created" simple_type="int" notinall="true"/>'."\n";
  if (!strpos($schema_content,'<field name="createdby"')) $result .= '<field name="createdby" simple_type="text" dbsize="20" notinall="true"/>'."\n";
  if (!strpos($schema_content,'<field name="lastmodified"')) $result .= '<field name="lastmodified" simple_type="int" notinall="true"/>'."\n";
  if (!strpos($schema_content,'<field name="lastmodifiedby"')) $result .= '<field name="lastmodifiedby" simple_type="text" dbsize="20" notinall="true"/>'."\n";
  if (!strpos($schema_content,'<field name="dsize"')) $result .= '<field name="dsize" simple_type="int" notinall="true"/>'."\n";
  if (!strpos($schema_content,'<field name="history"')) $result .= '<field name="history" simple_type="textarea" notinall="true"><onlyin views="history" /></field>'."\n";
  if (!strpos($schema_content,'<field name="folder"')) $result .= '<field name="folder" displayname="{t}Folder{/t}" simple_type="text" dbsize="15" simple_default="" hidden="true"><data values="@folder@"/></field>';
  if (!strpos($schema_content,'<field name="id"')) $result .= '<field name="id" simple_type="id" hidden="true"></field>'."\n";

  $matches = "";
  preg_match_all('|simple_tab="(.*?)"|msi',$schema_content,$matches);
  if (count($matches)>0) {
	foreach (array_unique($matches[1]) as $tab) {
	  if (!strpos($schema_content,'<tab name="'.$tab.'"')) $result .= '<tab name="'.$tab.'"/>'."\n";
	}
  }
  if (!preg_match("|<table[^>]*? disable_schema=\"true\".*?>|msi",$schema_content)) {
    $result .= '
	  <view name="schema" displayname="{t}Schema{/t}" schema="sys_nosql_schema" visibility="bottom" nosinglebuttons="true" noviewbuttons="true" template_mode="flat" tfield_1="filename" right="admin"></view>
    ';
  }
  if (!strpos($schema_content,'<tab name="general"')) $result .= '<tab name="general" displayname="{t}General{/t}" />'."\n";

  if (preg_match("|<table[^>]*? enable_new=\"true\".*?>|msi",$schema_content)) $result = '
	<viewbutton name="paste" displayname="{t}Paste{/t}" action="action_asset=paste" right="write" accesskey="v"/>
	<view name="edit_as_new" displayname="{t}Edit as new{/t}" template="edit" schema_mode="edit_as_new" nosinglebuttons="true" noviewbuttons="true" right="write" accesskey="r"></view>
	<view name="new" displayname="{t}New{/t}" template="edit" limit="1" default_sql="no_select" schema_mode="new" nosinglebuttons="true" noviewbuttons="true" right="write" accesskey="n"></view>
  '.$result;

  if (preg_match("|<table[^>]*? enable_new_only=\"true\".*?>|msi",$schema_content)) $result = '
	<viewbutton name="paste" displayname="{t}Paste{/t}" action="action_asset=paste" right="write" accesskey="v"/>
	<view name="new" displayname="{t}New{/t}" template="edit" limit="1" default_sql="no_select" schema_mode="new" nosinglebuttons="true" noviewbuttons="true" right="write" accesskey="n"></view>
  '.$result;

  if (!preg_match("|<table[^>]*? disable_copy=\"true\".*?>|msi",$schema_content)) $result = '
	<viewbutton name="copy" displayname="{t}Copy{/t}" action="action_asset=copy" accesskey="c"/>
  '.$result;
  
  if ((preg_match("|<table[^>]*? enable_purge=\"true\".*?>|msi",$schema_content) or
  	  preg_match("|<table[^>]*? enable_delete=\"true\".*?>|msi",$schema_content)) and 
	  !preg_match("|<table[^>]*? disable_cut=\"true\".*?>|msi",$schema_content)
  ) $result = '
	<viewbutton name="cut" displayname="{t}Cut{/t}" action="action_asset=cut" right="write" accesskey="x"/>
  '.$result;

  if (preg_match("|<table[^>]*? enable_purge=\"true\".*?>|msi",$schema_content)) $result .= '
	<viewbutton name="purge" displayname="{t}Delete{/t}" action="action_asset=purge" onclick="if (!confirm(\'{t}Really delete the dataset(s) ?{/t}\')) return false;" right="write" accesskey="d"/>
  ';
  if (preg_match("|<table[^>]*? enable_delete=\"true\".*?>|msi",$schema_content)) $result .= '
	<viewbutton name="delete" displayname="{t}Delete{/t}" action="action_asset=delete" onclick="if (!confirm(\'{t}Really delete the dataset(s) ?{/t}\')) return false;" right="write" accesskey="d"/>
  ';
  if (preg_match("|<table[^>]*? enable_edit=\"true\".*?>|msi",$schema_content)) $result = '
	<view name="edit" displayname="{t}Edit{/t}" schema_mode="edit" showinsingleview="true" right="write" enable_calendar="" accesskey="e"></view>
  '.$result;

  if (preg_match("|<table[^>]*? enable_empty=\"true\".*?>|msi",$schema_content)) $result .= '
	<viewbutton name="empty" displayname="{t}Empty folder{/t}" action="action_asset=empty" onclick="if (!confirm(\'{t}Really empty the folder ?{/t}\') || !confirm(\'{t}REALLY delete ALL datasets ?{/t}\')) return false;" right="write" accesskey="y"/>
  ';
  if (preg_match("|<table[^>]*? enable_purgeall=\"true\".*?>|msi",$schema_content)) $result .= '
	<viewbutton name="purgeall" displayname="{t}Empty folder{/t}" action="action_asset=purgeall" onclick="if (!confirm(\'{t}Really empty the folder ?{/t}\') || !confirm(\'{t}REALLY delete ALL datasets ?{/t}\')) return false;" right="write" accesskey="y"/>
  ';
  if (!preg_match("|<table[^>]*? disable_search=\"true\".*?>|msi",$schema_content)) {
    $result .= '
	  <view name="search" displayname="{t}Search{/t}" template="display" schema="sys_search" visibility="active" nosinglebuttons="true" noviewbuttons="true" nosqlfolder="true"></view>
    ';
  }
  $schema_content = str_replace("</table>",$result."</table>",$schema_content);

  if (!preg_match("|<table[^>]*? name=\".*?>|msi",$schema_content)) {
    $schema_content = str_replace("<table","<table name=\"simple_".($is_sys?"sys_":"").$schema."\"",$schema_content);
  }
  if (!preg_match("|<table[^>]*? modulename=\".*?>|msi",$schema_content)) {
    $schema_content = str_replace("<table","<table modulename=\"".ucfirst(($is_sys?"sys_":"").$schema)."\"",$schema_content);
  }

  $schema_content = preg_replace('|(<view [^>]*?)/>|msi',"\\1></view>",$schema_content);
  $schema_content = preg_replace('|(<view [^>]*?>)(.*?</view>)|msi',"\\1<sqlwhere value=\"id in (@item@)\"/>\\2",$schema_content);
  $schema_content = _xml_get_default_schema_validate($schema_content);
  return $schema_content;
}

function _xml_get_default_schema_validate($schema_content) {
  $schema_content = preg_replace('|(<field [^>]*?)/>|msi',"\\1></field>",$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? name="id"[^>]*?>)(.*?</field>)|msi',"\\1<KEY/>\\2",$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? name="folder"[^>]*?>)(.*?</field>)|msi',"\\1<INDEX/>\\2",$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? name="pid"[^>]*?>)(.*?</field>)|msi',"\\1<INDEX/>\\2",$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? simple_type="float"[^>]*?>)(.*?</field>)|msi','\\1<validate function="float"/>\\2',$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? simple_type="int"[^>]*?>)(.*?</field>)|msi','\\1<validate function="integer"/>\\2',$schema_content);

  $schema_content = preg_replace('|(<field[^>]*? simple_type="date"[^>]*?>)(.*?</field>)|msi','\\1<validate function="date"/><store function="datetime_to_int"/><restore function="dateformat||{t}m/d/Y{/t}"/>\\2',$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? simple_type="dateselect"[^>]*?>)(.*?</field>)|msi','\\1<validate function="date"/><store function="datetime_to_int"/><restore function="dateformat||{t}m/d/Y{/t}"/>\\2',$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? simple_type="time"[^>]*?>)(.*?</field>)|msi','\\1<validate function="time"/><store function="datetime_to_int"/><restore function="dateformat||{t}g:i a{/t}"/>\\2',$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? simple_type="datetime"[^>]*?>)(.*?</field>)|msi','\\1<validate function="datetime"/><store function="datetime_to_int"/><restore function="dateformat||{t}m/d/Y g:i a{/t}"/>\\2',$schema_content);

  $schema_content = preg_replace('|(<field[^>]*? simple_type="wikiarea"[^>]*?>)(.*?</field>)|msi','\\1<filter views="all" function="highlight_wiki"/>\\2',$schema_content);

  $schema_content = preg_replace('|(<field[^>]*? simple_type="checkbox"[^>]*?>)(.*?</field>)|msi','\\1<filter views="all" function="replacechecked"/><store function="replacechecked"/>\\2',$schema_content);
  $schema_content = preg_replace('|(<field[^>]*? simple_type="password"[^>]*?>)(.*?</field>)|msi','\\1<store function="sha1"/><validate function="password"/><filter views="all" function="hidepassword"/>\\2',$schema_content);

  $schema_content = preg_replace('|(<field[^>]*? simple_type="files"[^>]*?>)(.*?</field>)|msi','\\1<filter views="all" function="basename"/><validate function="checkvirus"/>\\2',$schema_content);

  return $schema_content;
}

function _xml_get_default_schema($schema_content,$schema,$is_sys=false) {
  $result = "";

  // database specific 
    
  if (!preg_match("|<table[^>]*? disable_rights=\"true\".*?>|msi",$schema_content)) {
    $result .= '
	  <view name="rights" displayname="{t}Rights{/t}" schema="sys_nosql_rights" visibility="active" nosinglebuttons="true" noviewbuttons="true" right="admin"></view>
	  <view name="rights_edit" displayname="{t}Edit rights{/t}" schema="sys_nosql_rights_edit" visibility="active" nosinglebuttons="true" noviewbuttons="true" schema_mode="edit" right="admin"></view>
    ';
  }
  if (!preg_match("|<table[^>]*? disable_history=\"true\".*?>|msi",$schema_content)) {
    $result .= '
	  <view name="history" displayname="{t}History{/t}" visibility="bottom" template="details" nosinglebuttons="true" noviewbuttons="true"></view>
    ';
  }

  if (!preg_match("|<table[^>]*? disable_structure=\"true\".*?>|msi",$schema_content)) {
    $result .= '
	  <view name="structure" displayname="{t}Structure{/t}" schema="sys_nosql_structure" visibility="bottom" nosinglebuttons="true" noviewbuttons="true" right="admin"></view>
    ';
  }
  if (!preg_match("|<table[^>]*? disable_index=\"true\".*?>|msi",$schema_content)) {
    $result .= '
	  <view name="index" displayname="{t}Index{/t}" schema="sys_nosql_index" visibility="bottom" nosinglebuttons="true" noviewbuttons="true" right="admin"></view>
    ';
  }

  if (!strpos($schema_content,'<field name="bgcolor"')) {
    $result .= '<rowfilter name="filter_sys0" views="all" simple_type="_bgstyle" function="buildbgcolor|bgcolor"/>'.
      '<field name="bgcolor" displayname="{t}Color{/t}" simple_type="select" simple_size="1" hidden="true" editable="true">'.
      '<data values="#DDDDFF_##_{t}blue{/t}|#CCFFCC_##_{t}green{/t}|#FFDDFF_##_{t}magenta{/t}|#FFDDAA_##_{t}orange{/t}|#FFCCCC_##_{t}red{/t}|#FFFFDD_##_{t}yellow{/t}"/>'.
	  '</field>'."\n";
  }
  $schema_content = str_replace("</table>",$result."</table>",$schema_content);

  return $schema_content;
}

function xml_file_get_contents($file,$schema) {
  $schema_content = file_get_contents($file);
  $schema_content = '<?xml version="1.0" encoding="utf-8"?><schema version="0.2">'."\n".substr($schema_content,strpos("@".$schema_content,"<table")-1)."</schema>";
  $schema_content = preg_replace("|<!--.*?-->|msi","",$schema_content);

  $is_sys = strpos($file,"_sys/");
  if (!sys_strbegins($schema,"nosql_")) {
	$schema_content = _xml_get_default_schema($schema_content,$schema,$is_sys);
  }
  $schema_content = _xml_get_default_schema_nosql($schema_content,$schema,$is_sys);
  if (DEBUG) $schema_content = sys_remove_trans($schema_content);
  return $schema_content;
}

function _xml_debug_schema($schema_content) {
  $i = 0;
  foreach (explode("\n",$schema_content) as $line) {
    $i++;
	echo $i." ".htmlspecialchars($line)."<br>";
  }
  echo "<br><br>";
}

function _xml_apply_indexes($schema_name,$schema_fields) {
  $buffer = array();
  if (($indexes = sql_table_get_indexes($schema_name)) === false) {
    $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_table_get_indexes").sql_error();
  }
  $primaries = array();
  foreach ($schema_fields as $field) {
	if (isset($field["INDEX"]) and (!isset($indexes[$field["NAME"]]) or !$indexes[$field["NAME"]]["index"])) {
	  if (!sql_add_index("", "ind_".str_replace(array("simple_"),"",$schema_name)."_".$field["NAME"], $schema_name, $field["NAME"])) {
	    $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_add_index - ".$schema_name." - ".$field["NAME"]).sql_error();
	  }
	}
	if (isset($field["INDEX_FULLTEXT"]) and (!isset($indexes[$field["NAME"]]) or !$indexes[$field["NAME"]]["index"])) {
	  if (!sql_add_index("FULLTEXT","ind_".str_replace("simple_","",$schema_name)."_".$field["NAME"],$schema_name,$field["NAME"])) {
	    $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_add_index_fulltext").sql_error();
	  }
	}
	if (isset($field["KEY"])) $primaries[] = $field["NAME"];
  }
  if (count($primaries)>0) {
    $change = false;
    foreach ($primaries as $primary) {
	  if (!isset($indexes[$primary]) or !$indexes[$primary]["primary"]) $change = true;
	}
	if ($change) {
	  sql_drop_index($schema_name,"PRIMARY KEY");
	  if (!sql_table_modify_primary($schema_name,implode(",",$primaries))) $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_table_modify_primary").sql_error();
	}
  }
  if (!sql_table_optimize($schema_name)) $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_table_optimize").sql_error();
  return $buffer;
}

function _xml_apply_schema($schema_name,$schema_fields) {
  $types = sql_types();
  $types = array(
    "text"=>$types["string"], "password"=>$types["string"], "checkbox"=>$types["bool"],
	"time"=>$types["date"], "date"=>$types["date"], "datetime"=>$types["date"],
	"id"=>$types["id"], "pid"=>$types["id"], "float"=>$types["float"], "int"=>$types["int"],
	"textarea"=>$types["text"], "htmlarea"=>$types["text"], "wikiarea"=>$types["text"], "spreadsheet"=>$types["text"],
	"chatarea"=>$types["string"], 
	"dateselect"=>$types["text"], "select"=>$types["text"], "multitext"=>$types["text"], 
	"files"=>$types["text"], "dateselect_small"=>$types["string"], "select_small"=>$types["string"], "files_small"=>$types["string"],
  );
  $buffer = array();
  if (!sql_table_exists($schema_name) and !sql_table_create($schema_name)) {
    $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_table_exists").sql_error();
  }
  foreach ($schema_fields as $field) {
	if (!isset($field["SIMPLE_TYPE"]) or isset($field["NODB"])) continue;
    $ftype = $field["SIMPLE_TYPE"];
	if ($ftype=="dateselect" and $field["SIMPLE_SIZE"]=="1") $ftype = "dateselect_small";
	if ($ftype=="select" and $field["SIMPLE_SIZE"]=="1") $ftype = "select_small";
	if ($ftype=="files" and $field["SIMPLE_SIZE"]=="1") $ftype = "files_small";
	$ftype = $types[$ftype];
	if (isset($field["DBSIZE"])) $ftype[1] = $field["DBSIZE"];
	if (sql_column_exists($schema_name,$field["NAME"])) $result = sql_table_modify($schema_name,$field["NAME"],sprintf($ftype[0],$ftype[1])); else $result = sql_table_add($schema_name,$field["NAME"],sprintf($ftype[0],$ftype[1]));
	if (!$result) $buffer[] = sprintf("{t}SQL FAILED: %s{/t}\n","sql_table_modify - ".$schema_name." - ".$field["NAME"]).sql_error();
  }
  $buffer = array_merge($buffer,_xml_apply_indexes($schema_name,$schema_fields));
  if (count($buffer)!=0) {
    sys_message_box(implode("\n\n",$buffer));
	return false;
  }
  return true;
}

function xml_parse_schema($data,$tname,$time,$return_view) {
  @set_time_limit(45);
  $parser = xml_parser_create("utf-8");
  xml_parser_set_option($parser,XML_OPTION_SKIP_WHITE,1);
  $values = "";
  $tags = "";
  xml_parse_into_struct($parser,$data,$values,$tags);
  xml_parser_free($parser);

  $tables = array();
  $extra_types = array("TAB","SINGLEBUTTON","VIEWBUTTON","VIEW","ROWFILTER");
  $i = 0;
  if (isset($tags["TABLE"]) and is_array($tags["TABLE"]) and count($tags["TABLE"])>0) {
    foreach ($tags["TABLE"] as $key=>$key2) {
	  $table_att_temp = array();
	  $table_fields_temp = array();
      if ($key % 2 == 0 and $values[$tags["TABLE"][$key]]["type"]=="open" and $values[$tags["TABLE"][$key+1]]["type"]=="close" and isset($values[$tags["TABLE"][$key]]["attributes"]["NAME"]) and $values[$tags["TABLE"][$key]]["attributes"]["NAME"]!="" and $tags["TABLE"][$key+1]-$tags["TABLE"][$key]>3) {
		$table_temp = array_slice($values,$tags["TABLE"][$key],$tags["TABLE"][$key+1]-$tags["TABLE"][$key]);
		$table_att_temp = $values[$tags["TABLE"][$key]]["attributes"];
		$table_field_marker_temp = "";
		
		foreach (array_keys($table_temp) as $key_2) {
		  if ($table_temp[$key_2]["tag"]=="FIELD") $i = 2;
		    elseif ($table_temp[$key_2]["tag"]=="VIEW") $i = 1;
		    elseif ($table_temp[$key_2]["tag"]=="TAB") $i = 3;
		    elseif ($table_temp[$key_2]["tag"]=="VIEWBUTTON") $i = 4;
		    elseif ($table_temp[$key_2]["tag"]=="SINGLEBUTTON") $i = 5;
		    elseif ($table_temp[$key_2]["tag"]=="ROWFILTER") $i = 6;

		  if ($i>0 and $table_temp[$key_2]["type"]=="complete" and isset($table_temp[$key_2]["attributes"]["NAME"]) and $table_temp[$key_2]["attributes"]["NAME"]!="" and (isset($table_temp[$key_2]["attributes"]["SIMPLE_TYPE"]) or in_array($table_temp[$key_2]["tag"],$extra_types))) {
		    $table_fields_temp[$i][$table_temp[$key_2]["attributes"]["NAME"]] = $table_temp[$key_2]["attributes"];

		  } elseif ($i>0 and $table_temp[$key_2]["type"]=="open" and isset($table_temp[$key_2]["attributes"]["NAME"]) and $table_temp[$key_2]["attributes"]["NAME"]!="" and (isset($table_temp[$key_2]["attributes"]["SIMPLE_TYPE"]) or in_array($table_temp[$key_2]["tag"],$extra_types))) {
		    $table_field_marker_temp = $table_temp[$key_2]["attributes"]["NAME"];
			$table_fields_temp[$i][$table_field_marker_temp] = $table_temp[$key_2]["attributes"];

		  } elseif ($i>0 and $table_temp[$key_2]["type"]=="close") {
		    $table_field_marker_temp = "";
			$i = 0;
		  } else {
		    if ($table_field_marker_temp!="" and $table_temp[$key_2]["type"]=="complete") {
			  if (isset($table_temp[$key_2]["attributes"]) and count($table_temp[$key_2]["attributes"])>0) {
			    $table_fields_temp[$i][$table_field_marker_temp][$table_temp[$key_2]["tag"]][] = $table_temp[$key_2]["attributes"];
			  } else {
			    $table_fields_temp[$i][$table_field_marker_temp][$table_temp[$key_2]["tag"]] = "";
			  }
			}
		  }
		}
	  }
	  if (count($table_fields_temp)>1 and count($table_att_temp)>0) {
	    $views = $table_fields_temp[1];
		$fields = $table_fields_temp[2];
		$tabs = (isset($table_fields_temp[3])?$table_fields_temp[3]:array("general"=>""));
		
		$buttons = (isset($table_fields_temp[4])?$table_fields_temp[4]:array());
		$singlebuttons = (isset($table_fields_temp[5])?$table_fields_temp[5]:array());
		$rowfilters = (isset($table_fields_temp[6])?$table_fields_temp[6]:array());

	    foreach ($fields as $key=>$field) {
		  if (isset($fields[$key]["ONLYIN"][0]["VIEWS"])) $fields[$key]["ONLYIN"] = explode("|",$fields[$key]["ONLYIN"][0]["VIEWS"]);
		  if (isset($fields[$key]["NOTIN"][0]["VIEWS"])) $fields[$key]["NOTIN"] = explode("|",$fields[$key]["NOTIN"][0]["VIEWS"]);
		  if (isset($fields[$key]["READONLYIN"][0]["VIEWS"])) $fields[$key]["READONLYIN"] = explode("|",$fields[$key]["READONLYIN"][0]["VIEWS"]);
		  if (isset($fields[$key]["SIMPLE_TYPE"]) and $fields[$key]["SIMPLE_TYPE"]=="id") $table_att_temp["ID"] = $key;
		  if (!isset($fields[$key]["SIMPLE_TAB"])) $fields[$key]["SIMPLE_TAB"] = key($tabs);
		  if (isset($fields[$key]["DATA"])) {
		    $values = array();
			$titles = array();
			foreach ($fields[$key]["DATA"] as $data_item) {
			  if (isset($data_item["VALUES"])) $values[] = $data_item["VALUES"];
			  if (isset($data_item["FUNCTION"])) $values[] = "#".str_replace("|","~",$data_item["FUNCTION"]);
			  if (isset($data_item["TITLE"])) $titles[] = $data_item["TITLE"]; else $titles[] = "";
			}
			$fields[$key]["DATA"] = $values;
			$fields[$key]["DATA_TITLE"] = $titles;
			
		  }
		  if (isset($fields[$key]["SIMPLE_DEFAULT_FUNCTION"]) and $fields[$key]["SIMPLE_DEFAULT_FUNCTION"]!="") {
			$fields[$key]["SIMPLE_DEFAULT"] = "|#".$fields[$key]["SIMPLE_DEFAULT_FUNCTION"]."#";
		  }
		  if (!isset($fields[$key]["SIMPLE_DEFAULT"])) $fields[$key]["SIMPLE_DEFAULT"] = "";
		  $fields[$key]["SIMPLE_DEFAULT"] = str_replace("\\n","\n",$fields[$key]["SIMPLE_DEFAULT"]);
		}
		if (!isset($table_att_temp["ID"])) $table_att_temp["ID"] = "id";
		if (!isset($table_att_temp["VERSION"])) $table_att_temp["VERSION"] = "1";

	    foreach ($views as $key=>$view) {
		  if (file_exists("ext/icons/".$view["NAME"].".gif")) $views[$key]["ICON"] = $view["NAME"].".gif";
		  $views[$key]["version"] = $table_att_temp["VERSION"];
		  $views[$key]["modulename"] = $table_att_temp["MODULENAME"];
		  $views[$key]["id"] = $table_att_temp["ID"];
		  $views[$key]["filters"] = array();
		  $views[$key]["restore_filters"] = array();
		  $views[$key]["rowfilters"] = array();
		  $views[$key]["fields"] = array();
		  $views[$key]["links"] = array();
		  $views[$key]["linkstext"] = array();
		  $views[$key]["tabs"] = $tabs;
		  $views[$key]["filters"] = array();

		  if (count($rowfilters)>0) {
		    foreach ($rowfilters as $rowfilter) {
			  $r_views = explode("|",$rowfilter["VIEWS"]);
			  if (in_array($key,$r_views) or in_array("all",$r_views)) {
			    $views[$key]["rowfilters"][] = $rowfilter;
			  }
			}
		  }

		  if (isset($view["SQLWHERE"])) {
		    $sql_where_strings = array();
		    foreach ($view["SQLWHERE"] as $sql_where) {
			  if (isset($sql_where["VALUE"]) and $sql_where["VALUE"]!="") {
				$sql_where_strings[]=$sql_where["VALUE"];
			  }
			}
			$views[$key]["SQLWHERE"] = $sql_where_strings;
		  }
		  foreach ($fields as $field) {
			$addfield = true;
			if (isset($field["NOTINALL"])) $addfield = false;
			if (isset($field["NOTIN"]) and in_array($key,$field["NOTIN"])) $addfield = false;
			if (isset($field["ONLYIN"])) {
			  if (in_array($key,$field["ONLYIN"])) $addfield = true; else $addfield = false;
			}
			if (isset($views[$key]["SHOWONLY"][0]["FIELDS"]) and !in_array($field["NAME"],explode("|",$views[$key]["SHOWONLY"][0]["FIELDS"]))) $addfield = false;

			if ($addfield) {
			  if (isset($field["READONLYIN"]) and in_array($key,$field["READONLYIN"])) {
			    $field["READONLYALL"] = "true";
			  }
			  $views[$key]["fields"][$field["NAME"]] = $field;
			  $tab = $field["SIMPLE_TAB"];
			  if (!isset($views[$key]["tabs"][$tab])) $views[$key]["tabs"][$tab] = $tabs[$tab];
			}

			if (!empty($field["LINK"][0]["VALUE"])) {
			  $link = $field["LINK"][0]["VALUE"];
			  if ($link[0]=="@") $link = array("_blank",substr($link,1)); else $link = array("",$link);
			  if (empty($field["LINK"][0]["VIEWS"])) {
			    $views[$key]["links"][$field["NAME"]] = $link;
			  } else {
			    $fviews = explode("|",$field["LINK"][0]["VIEWS"]);
			    if (in_array($key,$fviews) or in_array("all",$fviews)) $views[$key]["links"][$field["NAME"]] = $link;
			  }
			}

			if (!empty($field["LINKTEXT"][0]["VALUE"])) {
			  $link = $field["LINKTEXT"][0]["VALUE"];
			  if ($link[0]=="@") $link = array("_blank",substr($link,1)); else $link = array("",$link);
			  if (empty($field["LINKTEXT"][0]["VIEWS"])) {
			    $views[$key]["linkstext"][$field["NAME"]] = $link;
			  } else {
			    $fviews = explode("|",$field["LINKTEXT"][0]["VIEWS"]);
			    if (in_array($key,$fviews) or in_array("all",$fviews)) $views[$key]["linkstext"][$field["NAME"]] = $link;
			  }
			}
			if (isset($field["FILTER"])) {
			  foreach ($field["FILTER"] as $filter) {
			    $fviews = explode("|",$filter["VIEWS"]);
			    if (in_array($key,$fviews) or in_array("all",$fviews)) $views[$key]["filters"][$field["NAME"]][] = $filter;
			  }
			}
			if (isset($field["RESTORE"])) {
			  foreach ($field["RESTORE"] as $restore) {
			    if (isset($restore["VIEWS"])) {
			      $fviews = explode("|",$restore["VIEWS"]);
			      if (in_array($key,$fviews) or in_array("all",$fviews)) {
				    $views[$key]["restore_filters"][$field["NAME"]][] = $restore;
				  }
			    } else {
			      $views[$key]["restore_filters"][$field["NAME"]][] = $restore;
			    }
			  }
			}
		  }
	      if (!isset($view["SQL_HANDLER"])) $views[$key]["SQL_HANDLER"] = (isset($table_att_temp["SQL_HANDLER"])?$table_att_temp["SQL_HANDLER"]:"");
	      if (!isset($view["GROUP"])) $views[$key]["GROUP"] = (isset($table_att_temp["GROUP"])?$table_att_temp["GROUP"]:"");
	      if (!isset($view["DEFAULT_SQL"])) $views[$key]["DEFAULT_SQL"] = (isset($table_att_temp["DEFAULT_SQL"])?$table_att_temp["DEFAULT_SQL"]:"");
	      if (!isset($view["TEMPLATE"])) $views[$key]["TEMPLATE"] = (isset($table_att_temp["TEMPLATE"])?$table_att_temp["TEMPLATE"]:"");
	      if (!isset($view["SCHEMA_MODE"])) $views[$key]["SCHEMA_MODE"] = (isset($table_att_temp["SCHEMA_MODE"])?$table_att_temp["SCHEMA_MODE"]:"");
	      if (!isset($view["GROUPBY"])) $views[$key]["GROUPBY"] = (isset($table_att_temp["GROUPBY"])?$table_att_temp["GROUPBY"]:"");
	      if (!isset($view["ORDERBY"])) $views[$key]["ORDERBY"] = (isset($table_att_temp["ORDERBY"])?$table_att_temp["ORDERBY"]:$table_att_temp["ID"]);
	      if (!isset($view["ORDER"])) $views[$key]["ORDER"] = (isset($table_att_temp["ORDER"])?$table_att_temp["ORDER"]:"asc");
	      if (!isset($view["LIMIT"])) $views[$key]["LIMIT"] = (isset($table_att_temp["LIMIT"])?$table_att_temp["LIMIT"]:1);
	      if (!isset($view["ENABLE_CALENDAR"])) $views[$key]["ENABLE_CALENDAR"] = (isset($table_att_temp["ENABLE_CALENDAR"])?$table_att_temp["ENABLE_CALENDAR"]:"");
	      if (!isset($view["WHERE"]) or $view["WHERE"]=="") $views[$key]["WHERE"] = (isset($table_att_temp["WHERE"])?array($table_att_temp["WHERE"]):array());
		    else $views[$key]["WHERE"] = array($views[$key]["WHERE"]);
		  if (!isset($views[$key]["SQLWHERE"])) $views[$key]["SQLWHERE"] = array();
		  $views[$key]["SQLWHERE_DEFAULT"] = array();
		  $views[$key]["buttons"] = array();
		  $views[$key]["singlebuttons"] = array();

		  foreach ($buttons as $button_name=>$button) {
		    if (file_exists("ext/icons/".$button["NAME"].".gif")) $button["ICON"] = $button["NAME"].".gif";
		    $addit = true;
		    if (isset($button["VIEWS"]) and !in_array($key,explode("|",$button["VIEWS"]))) $addit = false;
			if ($addit) $views[$key]["buttons"][$button_name] = $button;
		  }
		  foreach ($singlebuttons as $button_name=>$button) {
		    if (file_exists("ext/icons/".$button["NAME"].".gif")) $button["ICON"] = $button["NAME"].".gif";
		    $addit = true;
		    if (isset($button["VIEWS"]) and !in_array($key,explode("|",$button["VIEWS"]))) $addit = false;
			if ($addit) $views[$key]["singlebuttons"][$button_name] = $button;
		  }
		  $views[$key]["nosingleview"] = array();

		  if (isset($view["BUTTON"])) {
			foreach ($view["BUTTON"] as $button) {
			  if (isset($buttons[$button["NAME"]])) $views[$key]["buttons"][$button["NAME"]] = $buttons[$button["NAME"]];
			}
		  }

		  if (isset($view["SINGLEBUTTONS"])) {
			$fbuttons = explode("|",$view["SINGLEBUTTONS"]);
			foreach ($views[$key]["singlebuttons"] as $key2=>$button) {
			  if (!in_array($key2,$fbuttons) and !in_array("all",$fbuttons)) unset($views[$key]["singlebuttons"][$key2]);
			}
		  }

		  if (isset($view["VIEWBUTTONS"])) {
			$fbuttons = explode("|",$view["VIEWBUTTONS"]);
			foreach ($views[$key]["buttons"] as $key2=>$button) {
			  if (!in_array($key2,$fbuttons) and !in_array("all",$fbuttons)) unset($views[$key]["buttons"][$key2]);
			}
		  }
		  
		  if (isset($view["NOVIEWBUTTON"][0])) {
			$fbuttons = explode("|",$view["NOVIEWBUTTON"][0]["NAMES"]);
			foreach ($fbuttons as $nobutton) {
			  if (isset($views[$key]["buttons"][$nobutton])) unset($views[$key]["buttons"][$nobutton]);
			}
		  }
		  if (isset($view["NOSINGLEBUTTON"][0])) {
			$fbuttons = explode("|",$view["NOSINGLEBUTTON"][0]["NAMES"]);
			foreach ($fbuttons as $nobutton) {
			  if (isset($views[$key]["singlebuttons"][$nobutton])) unset($views[$key]["singlebuttons"][$nobutton]);
			}
		  }
		  if (isset($view["NOSINGLEVIEW"][0]["VIEWS"])) {
		    foreach (explode("|",$view["NOSINGLEVIEW"][0]["VIEWS"]) as $view_item) {
			  $views[$key]["nosingleview"][$view_item] = "yes";
			}
		  }
          if (!isset($view["NOSQLWHERE"]) and count($views[$key]["WHERE"])>0) {
		    $views[$key]["SQLWHERE"][] = $views[$key]["WHERE"][0];
			$views[$key]["SQLWHERE_DEFAULT"][] = $views[$key]["WHERE"][0];
		  }
          if (isset($fields["folder"]) and !isset($view["NOSQLFOLDER"])) {
		    $views[$key]["SQLWHERE"][] = "folder in (@folders@)";
		    $views[$key]["SQLWHERE_DEFAULT"][] = "folder in (@folders@)";
		  }
	    }
	    $views2 = $views;
	    foreach ($views as $key=>$view) $views[$key]["views"] = $views2;

		if (!isset($table_att_temp["DEFAULT_VIEW"])) {
		  $table_att_temp["DEFAULT_VIEW"] = array_slice(array_keys($views),0,1);
		  $table_att_temp["DEFAULT_VIEW"] = $table_att_temp["DEFAULT_VIEW"][0];
  		}

		foreach ($views as $view_key=>$view) {
	      $tables[$view_key] = array("view"=>$view_key,"att"=>$table_att_temp,"views"=>array($view_key=>$view),"fields"=>$fields);
		}
	  	if (!sys_strbegins($tname,"nosql_") and !_xml_apply_schema($table_att_temp["NAME"],$fields)) sys_die("{t}Modifying database failed.{/t}");
	  }
 	  if (isset($table_att_temp["NAME"]) and (
	    !isset($table_att_temp["DEFAULT_VIEW"]) or !isset($views[$table_att_temp["DEFAULT_VIEW"]]) or count($fields)<1 
	  )) unset($tables[$table_att_temp["NAME"]]);
	  break;
    }
  }

  if (count($tables)>0) {
    foreach ($tables as $table_key=>$table) {
      $cache_file = SIMPLE_CACHE."/schema/".CORE_SGSML_VERSION."_".$tname."_".$table_key.".xml.ser";
      $fp = fopen($cache_file,"w+");
      fwrite($fp,serialize($table)); 
      fclose($fp);
	  touch($cache_file,$time);
    }

	$default_view = $table_att_temp["DEFAULT_VIEW"];
    $cache_file = SIMPLE_CACHE."/schema/".CORE_SGSML_VERSION."_".$tname."__default.xml.ser";
    $fp = fopen($cache_file,"w+");
    fwrite($fp,serialize($tables[$default_view]));
    fclose($fp);
	touch($cache_file,$time);

	/*
    $cache_file = SIMPLE_CACHE."/schema/".CORE_SGSML_VERSION."_".$tname.".xml.ser";
    $fp = fopen($cache_file,"w+");
    fwrite($fp,serialize($tables));
    fclose($fp);
	touch($cache_file,$time);
	*/
  } else {
    $cache_file = SIMPLE_CACHE."/schema/".CORE_SGSML_VERSION."_".$tname.".xml.ser";
	touch($cache_file,$time);
  }
  if ($return_view!="") {
    if (isset($tables[$return_view])) {
	  return $tables[$return_view];
	} else {
	  $GLOBALS["tview"] = array_shift(array_keys($tables));
	  return array_shift($tables);
	}
  }
  return $tables;
}  

/*  
function xml_export_values_to_xml($vals){
  $sep = -1;
  foreach($vals as $valobj){
    if($valobj['type']=='open' || $valobj['type']=='complete'){
	  $sep++;
      $retstr .= str_repeat("  ",$sep)."<".$valobj['tag'];
      if(is_array($valobj['attributes'])){ 
        foreach($valobj['attributes'] as $key=>$val){
          $retstr .= " $key=\"$val\"";
        }
      }
    }
    if($valobj['type']=='open'){
      $retstr .= ">".$valobj['value']."\n";
    }elseif($valobj['type']=='complete' && $valobj['value']){
      $retstr .= ">".$valobj['value']."</".$valobj['tag'].">\n";
	  $sep--;
    }elseif($valobj['type']=='complete'){
	  $sep--;
      $retstr .= " />\n";
    }elseif($valobj['type']=='cdata'){
      $retstr .= $valobj['value'];
    }elseif($valobj['type']=='close'){
      $retstr .= str_repeat("  ",$sep)."</".$valobj['tag'].">\n";
	  $sep--;
    }
  }
  return '<?xml version="1.0" encoding="utf-8"?>'."\n".$retstr;
}
*/
?>