<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

function fs_get_dirs($path, $left, $level, &$tree, $parent, $recursive) {
  if (validate_allowedpath($path)!="") return $left;
  $right = $left+1;
  if ($recursive and sys_is_folderstate_open($path,"fs",$parent)) {
    if (($handle=@opendir($path))) {
      while (false !== ($file = readdir($handle))) {
        if ($file=='.' or $file=='..' or !is_dir($path.$file)) continue;
		$right = fs_get_dirs($path.$file."/", $right, $level+1, $tree, $parent, true);
	  }
	}
  } else $right = $right+2;
  $tree[$left] = array("id"=>$path,"lft"=>$left,"rgt"=>$right,"flevel"=>$level,"ftitle"=>basename($path),"ftype"=>"sys_nosql_fs");
  return $right+1;
}

function fs_get_count($path,$where,$vars,$mfolder) {
  if (validate_allowedpath($path)!="") return 0;
  $files = 0;
  if (($handle=@opendir($path))) {
    while (false !== ($file = readdir($handle))) {
      if($file=='.' or $file=='..' or is_dir($path.$file)) continue;
	  $files++;
    }
    closedir($handle);
  }
  return $files;
}

function fs_get_select($path,$fields,$where,$order,$limit,$vars,$mfolder) {
  if (validate_allowedpath($path)!="") return array();
  $file_array = array();
  if (!$handle=@opendir($path)) return array();
  while (false !== ($file = readdir($handle))) {
    if($file=='.' or $file=='..' or is_dir($path.$file)) continue;
	$file_array[]=$file;
  }
  closedir($handle);

  $rows = array();
  foreach ($file_array as $filename) {
    $data = stat($path.$filename);
	$row = array();
	foreach ($fields as $field) {
	  switch ($field) {
	    case "filedata":
	    case "id": $row[$field] = $path.$filename; break;
	    case "folder": $row[$field] = $path; break;
	    case "filename":
		case "searchcontent": $row[$field] = $filename; break;
		case "fileext": $row[$field] = modify_getfileext($filename); break;
		case "fileatime": $row[$field] = $data["atime"]; break;
		case "created": $row[$field] = $data["ctime"]; break;
		case "lastmodified": $row[$field] = $data["mtime"]; break;
		case "lastmodifiedby": $row[$field] = ""; break;
		case "fileperms": $row[$field] = $data["mode"]; break;
		case "filesize": $row[$field] = $data["size"]; break;
		case "filecontent": $row[$field] = array("file"=>$path.$filename,"ext"=>modify_getfileext($filename),"size"=>$data["size"]); break;
	  }
	}
	if (sys_get_select_where($row,$where,$vars)) $rows[] = $row;
  }
  $rows = sys_get_select($rows,$order,$limit,$fields);
  if (count($rows)>0) {
    if (in_array("filecontent",$fields)) {
      foreach ($rows as $key=>$row) {
 		$rows[$key]["filecontent"] = modify_displayfile($row["filecontent"]["file"],$row["filecontent"]["size"],"file_get_contents");
	  }
	}
  }
  return $rows;
}

function fs_rename_folder($title,$folder) {
  $newpath = dirname($folder)."/".$title;
  if (validate_allowedpath(dirname($folder))!="" or @is_dir($newpath) or file_exists($newpath)) return "";
  rename($folder,$newpath);
  if (is_dir($newpath)) return "ok"; else return "";
}

function fs_create_folder($title,$parent) {
  $newpath = $parent.$title."/";
  if (validate_allowedpath($parent)!="" or is_dir($newpath) or file_exists($newpath)) return "";
  sys_mkdir($newpath);
  if (is_dir($newpath)) return "ok"; else return "";
}

function fs_delete_folder($path) {
  if (validate_allowedpath(dirname($path))!="" or !is_dir($path)) return "";
  dirs_delete_all($path);
  clearstatcache();
  if (!file_exists($path)) return "ok"; else return "";
}

function fs_get_delete($path,$where,$vars,$mfolder) {
  if (empty($vars["id"]) or validate_allowedpath(dirname($vars["id"]))!="" or !file_exists($vars["id"])) return "error";
  unlink($vars["id"]);
  if (!file_exists($vars["id"])) return ""; else return "error";
}

function fs_get_update($path,$data,$where,$vars,$mfolder) {
  if (empty($vars["id"])) return "error";
  if (validate_allowedpath($vars["folder"])!="" or !is_dir($vars["folder"])) return "";
  if (validate_allowedpath(dirname($vars["id"]))!="" or is_dir($vars["id"])) return "";
  if (file_exists($vars["id"])) {
    $target = $vars["folder"].basename($vars["id"]);
    if (file_exists($target)) unlink($target);
    rename($vars["id"],$target);
  }
  return "";
}

function fs_insert($path,$data,$mfolder) {
  if (validate_allowedpath(dirname($path))!="" or !is_dir($path)) return "";
  $source = $data["filedata"];
  $target = $path.modify_basename($data["filedata"]);
  if (!is_dir($source) and file_exists($source)) {
    if (file_exists($target)) unlink($target);
	rename($source,$target);
  }
}

?>