<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

function _icalendar_parse_date($val) {
  $val = str_replace(array("T","Z"),"",$val);
  if (!preg_match ("/([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})/", $val, $data)) return 0;
  if ($data[1]<1970) $data[1] = 1971;
  return mktime((int)$data[4], (int)$data[5], 0, (int)$data[2], (int)$data[3], (int)$data[1]);
}

function _icalendar_array_flat($arr) {
  if (count($arr)>0) return "|".implode("|",$arr)."|"; else return "";
}

function _icalendar_parse($file) {
  if ($data = sys_cache_get("icalendar_".sha1($file),ICALENDAR_CACHE)) return $data;
  if (validate_allowedpath(dirname($file))!="") return array();

  $rows = array();
  if (!$handle = fopen($file, "rb")) {
	$GLOBALS["t"]["notification"][] = sprintf("{t}Cannot read the file %s.{/t}",$file);
	return array();
  }
  $i = 0;
  $evopen = false;
  $lines = array();
  
    while (!feof($handle)) {
	  $line = fgets($handle, 10000);
	  $line = str_replace(array("\r","\n","\\n"),array("","","\n"),$line);
	  if ($line=="") continue;
	  if ($line[0]==" ") {
	    $lines[count($lines)-1] .= substr($line,1);
	  } else $lines[] = $line;
	}
    fclose($handle);

	foreach ($lines as $line) {
	  $line = trim($line);
	  switch($line) {
		case "BEGIN:VEVENT":
		  $i++;
	  	  $evopen = true;
		  $rows[$i] = array(
			"subject"=>"","begin"=>0,"ending"=>0,"created"=>0,"duration"=>0,"lastmodified"=>0,"allday"=>"0","location"=>"","description"=>"",
			"organizer"=>array(),"participants_ext"=>array(),"category"=>"","priority"=>"normal",
			"recurrence"=>"","repeatinterval"=>1,"repeatcount"=>"0","repeatuntil"=>0,"repeatexcludes"=>array()
		  );
		  break;
		case "BEGIN:VALARM":
	  	  $evopen = false;
		  break;
		case "END:VEVENT":
	  	  $evopen = false;
		  $begin = getdate($rows[$i]["begin"]);
		  $end = getdate($rows[$i]["ending"]);
		  if ($rows[$i]["lastmodified"]==0) $rows[$i]["lastmodified"] = $rows[$i]["created"];
		  if ($begin["hours"]==0 and $end["hours"]==0 and $begin["minutes"]==0 and $end["minutes"]==0) {
		    if ($rows[$i]["begin"]==$rows[$i]["ending"]) {
			  $rows[$i]["ending"] += 86399;
			} else $rows[$i]["ending"]--;
		    $rows[$i]["allday"] = "checked";
		  }
		  foreach ($rows[$i] as $key=>$item) {
			if (is_array($item)) $rows[$i][$key] = _icalendar_array_flat($item);
		  }
		  include_once("core/functions_trigger.php");
		  $rows[$i]["duration"] = $rows[$i]["ending"] - $rows[$i]["begin"];
		  $rows[$i] = array_merge($rows[$i],trigger_calcappointment("",$rows[$i]));
		  // $rows[$i]["begin_str"] = date("D d-m-y H:i:s",$rows[$i]["begin"]);
		  // $rows[$i]["end_str"] = date("D d-m-y H:i:s",$rows[$i]["ending"]);
		  break;
		default:
		  if (!$evopen) break;
		  $pos = strpos($line,":");
		  $first = substr($line,0,$pos);
		  $value_str = str_replace(array("\\,","\\n","\\N"),array(",","\n","\n"),substr($line,$pos+1));
		  if ($pos2 = strpos($first,";")) $kval = substr($first,0,$pos2); else $kval = $first;

		  switch($kval) {
			case "SUMMARY":
			  $rows[$i]["subject"] = $value_str; break;
			case "DESCRIPTION":
			  $rows[$i]["description"] = $value_str; break;
			case "LOCATION":
			  $rows[$i]["location"] = $value_str; break;
			case "CLASS":
			  if ($rows[$i]["category"]!="" and $value_str!="") $rows[$i]["category"] .= ",";
			  $rows[$i]["category"] .= ucfirst(strtolower($value_str)); break;		
			case "CATEGORIES":
			  if ($rows[$i]["category"]!="") $rows[$i]["category"] .= ",";
			  $rows[$i]["category"] .= $value_str; break;		
			case "UID":
			  $rows[$i]["id"] = $value_str; break;
			case "SEQUENCE":
			  $rows[$i]["sequence"] = $value_str; break;
			case "DTSTART":
			  $rows[$i]["begin"] = _icalendar_parse_date($value_str); break;
			case "DTEND":
			  $rows[$i]["ending"] = _icalendar_parse_date($value_str); break;
			case "LAST-MODIFIED":
			  $rows[$i]["lastmodified"] = _icalendar_parse_date($value_str); break;
			case "DTSTAMP":
			  $rows[$i]["created"] = _icalendar_parse_date($value_str); break;
			case "DURATION":
			  if (!preg_match("/PT?([0-9]{1,2}W)?([0-9]{1,2}D)?([0-9]{1,2}H)?([0-9]{1,2}M)?/",$value_str,$d)) break;
			  $d = array_merge($d,array(0,0,0,0));
			  $rows[$i]["ending"] = $rows[$i]["begin"] + str_replace("W","",$d[1])*604800 + str_replace("D","",$d[2])*86400 + str_replace("H","",$d[3])*3600 + str_replace("M","",$d[4])*60;
			  break;
			case "RRULE":
			  $value = explode(";",$value_str);
			  foreach ($value as $val) {
				$val = explode("=",$val);
				switch($val[0]) {
				  case "FREQ":
					switch ($val[1]) {
					  case "YEARLY": $rows[$i]["recurrence"] = "years"; break;
					  case "MONTHLY": $rows[$i]["recurrence"] = "months"; break;
					  case "WEEKLY": $rows[$i]["recurrence"] = "weeks"; break;
					  case "DAILY": $rows[$i]["recurrence"] = "days"; break;
					}
					break;
				  case "INTERVAL":
					$rows[$i]["repeatinterval"] = $val[1]; break;
				  case "COUNT":
					$rows[$i]["repeatcount"] = $val[1]; break;
				  case "UNTIL":
					$rows[$i]["repeatuntil"] = _icalendar_parse_date($val[1]); break;
				}
			  }
			  /*
			  echo "<pre>";
			  print_r($rows[$i]);
			  echo "</pre>";
			  echo date("D d-m-y H:i:s",$rows[$i]["begin"])."<br>\n".$line."<br>\n";
			  */
			  break;
			case "EXDATE":
			  $rows[$i]["repeatexcludes"][] = _icalendar_parse_date($value_str); break;
			case "ORGANIZER":
		  	  $value = explode(";",$value_str);
			  $key = explode(";",$first);
			  if (isset($value[1])) $value[0] = $value[1];
			  if (isset($key[1])) $value[0] = str_replace(array("CN=","\""),"",$key[1])." (".str_replace(array("MAILTO:","mailto:"),"",$value[0]).")";
			  $rows[$i]["organizer"][] = $value[0];
			  break;
			case "ATTENDEE":
	  		  $value = explode(";",$value_str);
			  $key = explode(";",$first);
			  if (isset($value[1])) $value[0] = $value[1];
			  $value[0] = str_replace(array("MAILTO:","mailto:"),"",$value[0]);
			  if (isset($key[1]) and strpos("@".$key[1],"CN=")>0) $value[0] = str_replace(array("CN=","\""),"",$key[1])." (".$value[0].")";
			  $rows[$i]["participants_ext"][] = $value[0];
			  break;
			default:
			  // echo $line."<br>\n";
			  break;
		  }
		  break;
	  }
	}

  sys_cache_set("icalendar_".sha1($file),$rows);
  return $rows;
}

function icalendar_get_dirs($path, $left, $level, &$tree, $parent, $recursive) {
  $right = $left+1;
  $tree[$left] = array("id"=>$path,"lft"=>$left,"rgt"=>$right,"flevel"=>$level,"ftitle"=>basename($path),"ftype"=>"nosql_".basename($path));
  return $right+1;
}

function icalendar_get_count($path,$where,$vars,$mfolder) {
  $count = count(_icalendar_parse($path));
  return $count;
}

function icalendar_get_select($path,$fields,$where,$order,$limit,$vars,$mfolder) {
  $path = rtrim($path,"/");
  $datas = _icalendar_parse($path);
  $rows = array();
  $i = 0;
  foreach ($datas as $data) {
	$i++;
    $row = array();
	foreach ($fields as $field) {
	  switch ($field) {
	    case "id": $row[$field] = $path."/?".$i; break;
	    case "folder": $row[$field] = $path; break;
		case "searchcontent": $row[$field] = implode(" ",$data); break;
	  	case "lastmodifiedby": $row[$field] = ""; break;
		default:
		  if (isset($data[$field])) {
			$row[$field] = $data[$field];
		  } else $row[$field] = "";
		  break;
	  }
	}
	if (sys_get_select_where($row,$where,$vars)) $rows[] = $row;
  }
  // fix for missing sql filtering
  $limit = array(0,count($rows));
  $rows = sys_get_select($rows,$order,$limit,$fields);
  return $rows;
}

?>