<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

include_once("lib/mail/IMAP.php");

function _imap_connect($mfolder) {
  $cid = "imap_".$mfolder;
  if (empty($GLOBALS["cache"][$cid."_conn"])) {
    if (isset($GLOBALS["credentials"][$cid])) {
      $username = $GLOBALS["credentials"][$cid]["username"];
	  $password = $GLOBALS["credentials"][$cid]["password"];
	  $server = $GLOBALS["credentials"][$cid]["server"];
	  $port = $GLOBALS["credentials"][$cid]["port"];
	  if (!$port) $port = 143;
	  $ssl = $GLOBALS["credentials"][$cid]["ssl"];
	  if ($ssl and !extension_loaded("openssl")) {
	    $GLOBALS["t"]["notification"][] = sprintf("[0] {t}%s is not compiled / loaded into PHP.{/t}","OpenSSL");
	    return false;
	  }
	} else return false;

	$imap = new Net_IMAP();
	if (PEAR::isError($result = $imap->connect(($ssl?$ssl."://".$server:$server), $port))) {
	  $GLOBALS["t"]["notification"][] = sprintf("[1] {t}Imap-conn-error: %s{/t}",$result->getMessage());
	  return false;
	}
	if (PEAR::isError($ret = $imap->login($username, $password))) {
	  $GLOBALS["t"]["notification"][] = sprintf("[2] {t}Imap-error: %s{/t}",$ret->getMessage());
	  return false;
	}
	
	$GLOBALS["cache"][$cid."_conn"] = $imap;
  }
  return $GLOBALS["cache"][$cid."_conn"];
}

function _imap_get_mailboxes($imap) {

  $boxes = $imap->getMailboxes();
  $delimiter = $imap->delimiter;
  if (count($boxes)>0) sort($boxes);
  if (count($boxes)>100) $boxes = array_slice($boxes,0,100);
  $mailboxes = array();
  foreach($boxes as $key=>$value) {
    $values = explode($delimiter,$value);
	$name = basename(array_pop($values));
	$parent = "";
	if ($name!="") {
	  if (count($values)>0) {
	    $parent = implode("/",$values);
	    $mailboxes[$parent][] = $name;
	    $parent .= "/";
	  } else $mailboxes["/"][] = $name;
	  $mailboxes[$parent.$name] = array();
	}
  }
  return $mailboxes;
}

function imap_get_dirs($path, $left, $level, &$tree, $parent, $recursive) {
  $right = $left+1;
  if ($recursive and sys_is_folderstate_open($path,"imap",$parent)) {
	$imap_path = substr($path,strpos($path,"/")+1,-1);
	if ($imap_path=="") $imap_path = "/";
	
    $cid = "imap_boxes_".md5($parent);
    if (!$imap_boxes = sys_cache_get($cid,IMAP_CACHE)) {
	  if (!$imap = _imap_connect($parent)) return $right+1;
	  $imap_boxes = _imap_get_mailboxes($imap);
      sys_cache_set($cid,$imap_boxes);
	}
	$mailboxes = $imap_boxes[$imap_path];
	if (count($mailboxes)>0) {
	  foreach($mailboxes as $mailbox) {
		$right = imap_get_dirs($path.$mailbox."/", $right, $level+1, $tree, $parent, true);
	  }
	}
  } else $right = $right+2;

  $tree[$left] = array("id"=>$path,"lft"=>$left,"rgt"=>$right,"flevel"=>$level,"ftitle"=>utf8_encode(basename($path)),"ftype"=>"sys_nosql_imap");
  return $right+1;
}

function imap_get_count($path,$where,$vars,$mfolder) {
  $imap_path = substr($path,strpos($path,"/")+1,-1);
  if ($imap_path=="") return 0;
  $cid = "imap_count_".md5($mfolder."/".$path);
  if ($count = sys_cache_get($cid,IMAP_LIST_CACHE)) return $count;
  if (!$imap = _imap_connect($mfolder)) return 0;
  if (PEAR::isError($count = $imap->getNumberOfMessages($imap_path))) return 0;
  sys_cache_set($cid,$count);
  return $count;
}

function _imap_get_datas($mfolder,$imap_path,$cid) {
  if ($datas = sys_cache_get($cid,IMAP_LIST_CACHE)) return $datas;
  if (!$imap = _imap_connect($mfolder)) return array();
  if (PEAR::isError($result = $imap->selectMailbox($imap_path))) {
	$datas = array(array(),array());
  } else {
    $datas = array();
    $datas[0] = $imap->getParsedHeaders("1:*","");
    $datas[1] = $imap->getListing();
  }
  sys_cache_set($cid,$datas);
  return $datas;
}

function _imap_getstructure($msg_id,$mfolder,$imap_path,$cid) {
  if ($structure = sys_cache_get($cid,IMAP_MAIL_CACHE)) return $structure;
  if (!$imap = _imap_connect($mfolder)) return array();
  if (PEAR::isError($result = $imap->selectMailbox($imap_path))) {
	$structure = array();
  } else {
	$structure = $imap->getStructure($msg_id);
  }
  sys_cache_set($cid,$structure);
  return $structure;
}

function _imap_getmessage($msg_id,$msg_uid,$mfolder,$imap_path,$cid) {
  if ($message = sys_cache_get($cid,IMAP_MAIL_CACHE)) return $message;
  if (!$imap = _imap_connect($mfolder)) return array();
  $message = "";
  if (!PEAR::isError($result = $imap->selectMailbox($imap_path))) {
    $structure = _imap_getstructure($msg_id,$mfolder,$imap_path,"imap_structure_".md5($msg_uid));
	foreach ($structure as $item) {
	  $content = "";
	  switch($item["contenttype"]) {
	    case "message/rfc822":
	      $headers = $imap->getParsedHeaders($msg_id,$item["id"]?$item["id"].".":"");
		  $content .= _imap_drawheader($headers[0]);
		  break;
		default:
		  if ($item["disposition"]!="attachment" and $item["size"]>0 and strpos("@".$item["contenttype"],"text/")) {
			$data_body = $imap->getBodyPart($msg_id,$item["id"],$item["encoding"]);
			if ($content!="") $content .= "\n";
			if ($item["contenttype"]=="text/html") {
			  $data_body = modify_htmlmessage($data_body);
			} else if ($item["contenttype"]!="text/plain") {
			  $content .= "[".$item["contenttype"]."]\n";
			}
			$content .= trim($data_body)."\n";
		  }
		  break;
	  }
	  if ($item["charset"]!="utf-8" and (strpos("@".$item["charset"],"iso") or strpos("@".$item["charset"],"ascii"))) $content = utf8_encode($content);
	  if ($message!="" and $content!="") $content = "\n----\n\n".$content;
	  $message .= $content;
 	} 
  }
  sys_cache_set($cid,$message);
  return $message;
}

/* message_html
			if (empty($structure)) $structure = $imap->getStructure($msg_id);
			foreach ($structure as $item) {
			  $content = "";
			  switch($item["contenttype"]) {
			    case "message/rfc822":
			   	  $headers = $imap->getParsedHeaders($msg_id,$item["id"]?$item["id"].".":"");
				  if ($content!="") $content .= "<hr>";
				  $content .= "<div style='margin-left:".($item["flevel"]*20)."px;'>";
				  $content .= "Subject: ".htmlspecialchars($headers["subject"])."<br>Date: ".htmlspecialchars($headers["date"])."<br>";
				  $content .= "From: ".htmlspecialchars($headers["from"])."<br>To: ".htmlspecialchars($headers["to"])."</div>";
				  break;
				default:
				  if ($item["disposition"]!="attachment" and $item["size"]>0) {
					$data_body = $imap->getBodyPart($msg_id,$item["id"],$item["encoding"]);
					if ($content!="") $content .= "<hr>";
					if ($item["contenttype"]!="text/html") {
					  $data_body = nl2br(htmlspecialchars(trim($data_body)));
					  if ($item["contenttype"]!="text/plain") $content .= "<b>[".$item["contenttype"]."]</b><br>";
					}
			        $content .= "<div style='margin-left:".($item["flevel"]*20)."px;'><code>".$data_body."</code></div>";
				  }
				  break;
			  }
			  if ($item["charset"]!="utf-8" and (strpos("@".$item["charset"],"iso") or strpos("@".$item["charset"],"ascii"))) $content = utf8_encode($content);
			  $row[$field] .= $content;
			}
*/

function _imap_drawheader($headers) {
  $content = "Subject: ".$headers["subject"]."\n";
  $content .= "Date: ".$headers["date"]."\n";
  $content .= "From: ".$headers["from"]."\n";
  $content .= "To: ".$headers["to"]."\n";
  return $content;
}

function imap_get_select($path,$fields,$where,$order,$limit,$vars,$mfolder) {
  $imap_path = substr($path,strpos($path,"/")+1,-1);
  if ($imap_path=="") return array();

  $cid = "imap_".md5($mfolder."/".$path);
  $datas = _imap_get_datas($mfolder,$imap_path,$cid);
  $rows = array();
  foreach ($datas[0] as $key=>$data) {
    $msg_id = $datas[1][$key]["msg_id"];
	$msg_uid = $datas[1][$key]["uidl"];
	$structure = array();
	$row = array();
	foreach ($fields as $field) {
	  switch ($field) {
	      case "id": $row[$field] = $path."/?".md5($msg_uid); break;
	      case "folder": $row[$field] = $path; break;
  		  case "searchcontent": $row[$field] = $data["subject"]." ".$data["from"]; break;
		  case "subject": $row[$field] = isset($data["subject"])?$data["subject"]:""; break;
	 	  case "efrom": $row[$field] = isset($data["from"])?$data["from"]:""; break;
	      case "eto": $row[$field] = isset($data["to"])?$data["to"]:""; break;
  		  case "cc": $row[$field] = isset($data["cc"])?$data["cc"]:""; break;
		  case "attachments": $row[$field] = ""; break;
		  case "attachments_show": $row[$field] = ""; break;
		  case "message": $row[$field] = ""; break;
	  	  case "created":
	      case "lastmodified": $row[$field] = isset($data["date"])?strtotime($data["date"]):"0"; break;
	  	  case "lastmodifiedby": $row[$field] = ""; break;
		  // Answered Flagged Deleted Seen Draft
		  case "flags": $row[$field] = strtolower(str_replace("\\","",implode(" ",$datas[1][$key]["flags"]))); break;
	  	  case "dsize":
		  case "size": $row[$field] = $datas[1][$key]["size"]; break;
		  case "headers":
  		    $row[$field] = "";
	  	    foreach ($data as $data_key=>$data_item) $row[$field] .= ucfirst($data_key).": ".(is_array($data_item)?implode("\n  ",$data_item):$data_item)."\n";
		    break;
	  }
	}
	$row["_msg_id"] = $msg_id;
	$row["_msg_uid"] = $msg_uid;
	$row["seen"] = isset($_SESSION["seen"]["imap:".$path."/?".md5($msg_uid)])?"checked":"";
	if (sys_get_select_where($row,$where,$vars)) $rows[] = $row;
  }

  $rows = sys_get_select($rows,$order,$limit,$fields);
  if (count($rows)>0) {
    foreach ($rows as $key=>$row) {
  	  $structure = "";
      if (in_array("message",$fields)) {
		$rows[$key]["message"] = _imap_getmessage($row["_msg_id"],$row["_msg_uid"],$mfolder,$imap_path,"imap_message_".md5($row["_msg_uid"]));
	  }			
      if (in_array("attachments_show",$fields)) {
		$rows[$key]["attachments_show"] = "";
		$files = array();
	    $structure = _imap_getstructure($row["_msg_id"],$mfolder,$imap_path,"imap_structure_".md5($row["_msg_uid"]));
	    foreach ($structure as $item) {
		  if ($item["disposition"]=="attachment") {
		    $files[] = $item["name"]." (".modify_filesize($item["size"]).")";
		  }
	    }
	    if (count($files)>0) $rows[$key]["attachments_show"] = "|".implode("|",$files)."|";
	  }
      if (in_array("attachments",$fields)) {
		$rows[$key]["attachments"] = "";
	    $structure = _imap_getstructure($row["_msg_id"],$mfolder,$imap_path,"imap_structure_".md5($row["_msg_uid"]));
	    $files = array();
		foreach ($structure as $item) {
		  if ($item["disposition"]=="attachment") {
	        $local = SIMPLE_CACHE."/imap/".sha1($row["_msg_uid"])."--".urlencode($item["name"]);
		    if (!file_exists($local) and $imap=_imap_connect($mfolder)) {
  	  		  $imap->selectMailbox($imap_path);
			  $imap->getBodyPart($row["_msg_id"],$item["id"],$item["encoding"],$local);
			}
 		    $files[] = $local;
		  }
		}
		if (count($files)>0) $rows[$key]["attachments"] = "|".implode("|",$files)."|";
	  }
/*
      if (in_array("data",$fields)) {
	    $local = SIMPLE_CACHE."/imap/".sha1($row["id"])."--"."email.txt";
	    if (!file_exists($local) and $imap=_imap_connect($mfolder)) {
		  $imap->selectMailbox($imap_path);
		  $imap->getBodyPart($row["_msg_id"],"","",$local);
		}
	    $rows[$key]["data"] = $local;
	  }
*/
	}
  }
  return $rows;
}

function imap_get_delete($path,$where,$vars,$mfolder) {
  if (empty($vars["id"])) return "error";
  
  $imap_path = substr($path,strpos($path,"/")+1,-1);
  if ($imap_path=="") return "error";

  if (!$imap = _imap_connect($mfolder)) return "error";
  if (PEAR::isError($result = $imap->selectMailbox($imap_path))) {
	$GLOBALS["t"]["notification"][] = sprintf("[5] {t}Imap-conn-error: %s{/t}",$result->getMessage());
	return array();
  }
  $datas = $imap->getListing();
  foreach ($datas as $key=>$data) {
    if ($vars["id"]==$path."/?".md5($data["uidl"])) {
  	  if (sys_strbegins($imap_path."/","Trash/") or sys_strbegins($imap_path."/","INBOX/Trash/")) {
  	    if (PEAR::isError($result = $imap->deleteMsg($data["msg_id"]))) {
	      $GLOBALS["t"]["notification"][] = sprintf("[6] {t}Imap-error: %s{/t}",$result->getMessage());
	      return "error";
	    }
	  } else {
	    if (PEAR::isError($result = $imap->copyMessages("Trash", $data["msg_id"]))) {
	      if (PEAR::isError($result = $imap->copyMessages("INBOX.Trash", $data["msg_id"]))) {
	        $GLOBALS["t"]["notification"][] = sprintf("[7] {t}Imap-error: %s{/t}",$result->getMessage());
	        return "error";
		  }
	    }
  	    if (PEAR::isError($result = $imap->deleteMsg($data["msg_id"]))) {
	      $GLOBALS["t"]["notification"][] = sprintf("[8] {t}Imap-error: %s{/t}",$result->getMessage());
	      return "error";
	    }
	  }
	  break;
    }
  }
  if (PEAR::isError($result = $imap->cmdExpunge())) {
    $GLOBALS["t"]["notification"][] = sprintf("[9] {t}Imap-error: %s{/t}",$result->getMessage());
	return "error";
  }
  sys_cache_remove("imap_".md5($mfolder."/".$path));
  sys_cache_remove("imap_count_".md5($mfolder."/".$path));
  return "";
}

function imap_get_update($path,$data,$where,$vars,$mfolder) {
  if (empty($vars["id"])) return "error";
  
  if (isset($data["seen"])) {
	$_SESSION["seen"]["imap:".$vars["id"]] = "";
	if (count($_SESSION["seen"])>100) $_SESSION["seen"] = array_slice($_SESSION["seen"],-100);
	return "";
  }
  
  $imap_path = substr($path,strpos($path,"/")+1,-1);
  if ($imap_path=="") return "error";

  $source_path = substr($vars["folder_source"],strpos($vars["folder_source"],"/")+1,-1);
  if ($source_path=="") return "error";

  if (!$imap = _imap_connect($mfolder)) return "error";

  if (PEAR::isError($result = $imap->selectMailbox($source_path))) {
	$GLOBALS["t"]["notification"][] = sprintf("[10] {t}Imap-conn-error: %s{/t}",$result->getMessage());
	return "error";
  }
  
  if (PEAR::isError($datas = $imap->getListing())) {
	$GLOBALS["t"]["notification"][] = sprintf("[11] {t}Imap-conn-error: %s{/t}",$datas->getMessage());
	return "error";
  }
  foreach ($datas as $key=>$data) {
    if (strpos("@".$vars["id"],$source_path."//?".md5($data["uidl"]))) {
	  if (PEAR::isError($result = $imap->copyMessages(str_replace("/",$imap->delimiter,trim($imap_path,"/")), $data["msg_id"]))) {
	    $GLOBALS["t"]["notification"][] = sprintf("[12] {t}Imap-error: %s{/t}",$result->getMessage());
	    return "error";
	  }
  	  if (PEAR::isError($result = $imap->deleteMsg($data["msg_id"]))) {
	    $GLOBALS["t"]["notification"][] = sprintf("[13] {t}Imap-error: %s{/t}",$result->getMessage());
	    return "error";
	  }
	  break;
	}
  }
  if (PEAR::isError($result = $imap->cmdExpunge())) {
    $GLOBALS["t"]["notification"][] = sprintf("[14] {t}Imap-error: %s{/t}",$result->getMessage());
	return "error";
  }
  sys_cache_remove("imap_".md5($mfolder."/".$source_path."/"));
  sys_cache_remove("imap_count_".md5($mfolder."/".$source_path."/"));
  sys_cache_remove("imap_".md5($mfolder."/".$path));
  sys_cache_remove("imap_count_".md5($mfolder."/".$path));
  return "";
}

function _imap_encodeHeaders($input) {
  foreach ($input as $hdr_name => $hdr_value) {
    preg_match_all("/(\w*[\x80-\xFF]+\w*)/", $hdr_value, $matches);
    foreach ($matches[1] as $value) {
      $replacement = preg_replace("/([\x80-\xFF])/e",'"=".strtoupper(dechex(ord("\1")))',$value);
      $hdr_value = str_replace($value,"=?UTF-8?Q?".$replacement."?=",$hdr_value);
    }
    $input[$hdr_name] = $hdr_value;
  }
  return $input;
}

function imap_insert($path,$data,$mfolder) {
  if (empty($data["efrom"]) or $data["efrom"]=="none") return "{t}No sender specified.{/t}";

  $imap_path = substr($path,strpos($path,"/")+1,-1);
  if ($imap_path=="") return "error".$path;

  if (!$imap = _imap_connect($mfolder)) return "error";

  if (PEAR::isError($result = $imap->selectMailbox($imap_path))) {
	$GLOBALS["t"]["notification"][] = sprintf("[15] {t}Imap-conn-error: %s{/t}",$result->getMessage());
	return "error";
  }
  
  $email = array();
  if ($data["attachments"]!="") {
	@set_time_limit(120); // 2min
    include_once("lib/mail/mimePart.php");

    $email = new Mail_mimePart("This is a multi-part message in MIME format.", array("content_type"=>"multipart/mixed"));
    $email->addSubPart($data["message"], array("content_type"=>"text/plain","charset"=>"UTF-8","encoding"=>"7bit"));
    $attachments = explode("|",$data["attachments"]);
	foreach ($attachments as $attachment) {
	  if (file_exists($attachment) and filesize($attachment)>0) {
	    $mime = "application/octet-stream";
	  	switch (modify_getfileext($attachment)) {
		  case "gif": $mime = "image/gif"; break;
		  case "jpg": $mime = "image/jpeg"; break;
		  case "png": $mime = "image/png"; break;
		  case "txt": $mime = "text/plain"; break;
		  case "html": case "htm": $mime = "text/html"; break;
		}
	    $email->addSubPart(file_get_contents($attachment), array(
		  "content_type"=>$mime."; name=\"".modify_basename($attachment)."\"", "encoding"=>"base64",
		  "disposition"=>"attachment", "dfilename"=>modify_basename($attachment),
	    ));
	  }
	}
	$email = $email->encode();
  }

  $headers = array(
	"Subject: ".$data["subject"], "From: ".$data["efrom"],
    "To: ".$data["eto"], "Date: ".date("r",$GLOBALS["time"]), 
    "Mime-Version: 1.0", "X-Mailer: Simple Groupware ".CORE_VERSION,
    "Content-Type: ".($data["attachments"]?$email["headers"]["Content-Type"]:"text/plain; charset=UTF-8")
  );
  $headers = _imap_encodeHeaders($headers);

  @set_time_limit(120); // 2min.
  $message = implode("\r\n",$headers)."\r\n\r\n".($data["attachments"]?$email["body"]:$data["message"]);

  if (PEAR::isError($result = $imap->appendMessage($message))) {
    $GLOBALS["t"]["notification"][] = sprintf("[16] {t}Imap-error: %s{/t}",$result->getMessage());
	return "error";
  }
  sys_cache_remove("imap_".md5($mfolder."/".$path));
  sys_cache_remove("imap_count_".md5($mfolder."/".$path));
  return "";
}
?>