<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

function _ldap_connect($mfolder) {
  $cid = "ldap_".$mfolder;
  if (empty($GLOBALS["cache"][$cid."_conn"])) {
    if (isset($GLOBALS["credentials"][$cid])) {
      $username = $GLOBALS["credentials"][$cid]["username"];
	  $password = $GLOBALS["credentials"][$cid]["password"];
	  $server = $GLOBALS["credentials"][$cid]["server"];
	  $port = $GLOBALS["credentials"][$cid]["port"];
	  $basedn = $GLOBALS["credentials"][$cid]["options"];
	  if (!$port) $port = 389;
	  $ssl = $GLOBALS["credentials"][$cid]["ssl"];
	  if ($ssl and !extension_loaded("openssl")) {
	    $GLOBALS["t"]["notification"][] = sprintf("[0] {t}%s is not compiled / loaded into PHP.{/t}","OpenSSL");
	    return false;
	  }
	} else return false;

  	if (!function_exists("ldap_connect")) {
	  $GLOBALS["t"]["notification"][] = sprintf("{t}%s is not compiled / loaded into PHP.{/t}","LDAP");
	  return false;
	}
    if (!$ds=ldap_connect($server)) sys_die(sprintf("{t}LDAP connection to host %s failed.{/t}",$server));
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3) ;
	
    if (!@ldap_bind($ds, $username, $password)) {
      if (!@ldap_bind($ds)) {
	    $GLOBALS["t"]["notification"][] = "{t}LDAP anonymous connection failed.{/t}";
	    return false;
	  }
	  if ($basedn=="") {
        $result_id = @ldap_read($ds,"","(objectclass=*)",array("namingContexts"));
  	    $attrs = ldap_get_attributes($ds, ldap_first_entry($ds,$result_id));
		if (isset($attrs["namingContexts"]) and is_array($attrs["namingContexts"])) {
		  $basedn = $attrs["namingContexts"][0];
		}
	  }
  	  $res = ldap_search($ds,$basedn,"uid=".$username);
  	  if (ldap_count_entries($ds,$res)==1) {
    	$dn = ldap_get_dn($ds, ldap_first_entry($ds,$res));
    	if (@ldap_bind($ds, $dn, $password)) {
		  $GLOBALS["t"]["notification"][] = sprintf("{t}Login failed from %s.{/t} (ldap) (%s)\n{t}(for active directory username must be: username@domain){/t}",_login_get_remoteaddr(),ldap_error($ds));
		  return false;
		}
	  }
	}
	$GLOBALS["cache"][$cid."_conn"] = $ds;
  }
  return $GLOBALS["cache"][$cid."_conn"];
}

function _ldap_base_dn($mfolder) {
  $cid = "ldap_".md5("basedn_".$mfolder);
  if ($basedn = sys_cache_get($cid,LDAP_LIST_CACHE)) return $basedn;

  $basedn = "";
  $cid2 = "ldap_".$mfolder;
  if (isset($GLOBALS["credentials"][$cid2])) $basedn = $GLOBALS["credentials"][$cid2]["options"];

  if ($basedn=="") {
    if (!$ds = _ldap_connect($mfolder)) return "";
    $result_id = ldap_read($ds,"","(objectclass=*)",array("namingContexts"));
    $attrs = ldap_get_attributes($ds, ldap_first_entry($ds,$result_id));
	if (isset($attrs["namingContexts"]) and is_array($attrs["namingContexts"])) {
      $basedn = $attrs["namingContexts"][0];
	}
  }
  sys_cache_set($cid,$basedn);
  return $basedn;
}

function ldap_get_dirs($path, $left, $level, &$tree, $parent, $recursive) {
  $right = $left+1;
  $ldap_path = substr($path,strpos($path,"/")+1,-1);
  if ($recursive and sys_is_folderstate_open($path,"ldap",$parent)) {
	if ($ldap_path!="") $paths = array_reverse(explode("/",$ldap_path)); else $paths = array();
	$paths[] = _ldap_base_dn($parent);
	$new_path = implode(",",$paths);

    $cid = "ldap_".md5("boxes_".$parent."/".$path);
    if (!$info = sys_cache_get($cid,LDAP_LIST_CACHE)) {
	  if (!$ds = _ldap_connect($parent)) return $right+1;
  	  if (!$sr = @ldap_list($ds,$new_path,"objectClass=*",array("dn","objectClass"))) return $right+1;
      $info = ldap_get_entries($ds, $sr);
	  sys_cache_set($cid,$info);
	}
	foreach ($info as $item) {
	  if (!is_array($item)) continue;
	  if (isset($item["objectclass"]) and 
	  	(in_array("organizationalPerson",$item["objectclass"]) or in_array("organizationalRole",$item["objectclass"]) or 
		 in_array("posixAccount",$item["objectclass"]) or in_array("inetOrgPerson",$item["objectclass"]) or 
	  	 in_array("group",$item["objectclass"]))) continue;
      $data = ldap_explode_dn($item["dn"],0);
	  $right = ldap_get_dirs($path.$data[0]."/", $right, $level+1, $tree, $parent, true);
	}
  }
  $tree[$left] = array("id"=>$path,"lft"=>$left,"rgt"=>$right,"flevel"=>$level,"ftitle"=>utf8_encode(basename($path)),"ftype"=>"sys_nosql_ldap");
  return $right+1;
}

function ldap_get_count($path,$where,$vars,$mfolder) {
  $ldap_path = substr($path,strpos($path,"/")+1,-1);
  if ($ldap_path!="") $paths = array_reverse(explode("/",$ldap_path)); else $paths = array();
  $paths[] = _ldap_base_dn($mfolder);
  $path = implode(",",$paths);
  $cid = "ldap_".md5("count_".$mfolder."/".$path);
  if ($count = sys_cache_get($cid,LDAP_LIST_CACHE)) return $count;
  if (!$ds = _ldap_connect($mfolder)) return 0;
  if (!$sr = @ldap_list($ds,$path,"objectClass=*",array("dn"))) return 0;
  $count = ldap_count_entries($ds,$sr);
  sys_cache_set($cid,$count);
  return $count;
}

function ldap_get_select($path,$fields,$where,$order,$limit,$vars,$mfolder) {
  $ldap_path = substr($path,strpos($path,"/")+1,-1);
  if ($ldap_path!="") $paths = array_reverse(explode("/",$ldap_path)); else $paths = array();
  $paths[] = _ldap_base_dn($mfolder);
  $new_path = implode(",",$paths);

  $mapping = array(
    "lastname"=>"sn",
    "firstname"=>"givenname",
	"email"=>"mail",
	"phone"=>"telephonenumber",
	"street"=>"postaladdress"
  );
  $cid = "ldap_".md5($mfolder."/".$path);
  if (!$datas = sys_cache_get($cid,LDAP_LIST_CACHE)) {
    $datas = array();
    if (!$ds = _ldap_connect($mfolder)) return array();
    if (!$sr = @ldap_list($ds,$new_path,"objectClass=*",array("dn","objectClass"))) {
	  $GLOBALS["t"]["notification"][] = ldap_error($ds);
	  return array();
	}
    $info = ldap_get_entries($ds, $sr);
    foreach ($info as $item) {
	  if (!is_array($item)) continue;
	  $sr = ldap_read($ds,$item["dn"],"objectClass=*",array("*","modifyTimestamp","modifiersName","createTimestamp","creatorsName"));
      $data = ldap_get_entries($ds, $sr);
	  if (empty($data[0])) continue;
	  $data = $data[0];
	  $datas[] = $data;
    }
	sys_cache_set($cid,$datas);
  }
  
  $rows = array();
  foreach ($datas as $data) {
    $row = array();
    foreach ($fields as $field) {
	  switch ($field) {
	     case "id": $row[$field] = $path."/?".md5($data["dn"]); break;
	     case "folder": $row[$field] = $path; break;
		 case "contactid":
      	   $data_2 = ldap_explode_dn($data["dn"],0);
		   $row[$field] = $data_2[0];
		   break;
		 case "lastmodified":
		   $row[$field] = 0; // TODO2 parse $data["modifytimestamp"][0] 20060312043657Z
		   break;
		 case "created":
		   $row[$field] = 0; // TODO2 parse $data["createtimestamp"][0]
		   break;
		 case "lastmodifiedby":
      	   $data_2 = ldap_explode_dn($data["modifiersname"][0],0);
		   $row[$field] = $data_2[0];
		   break;
		 case "createdby":
      	   $data_2 = ldap_explode_dn($data["creatorsname"][0],0);
		   $row[$field] = $data_2[0];
		   break;
		 default: 
		   $row[$field] = "";
		   if (isset($mapping[$field])) $ld_field = $mapping[$field]; else $ld_field = $field;
		   if (isset($data[$ld_field][0])) $row[$field] = $data[$ld_field][0];
		   break;
	  }
	}
    if (sys_get_select_where($row,$where,$vars)) $rows[] = $row;
  }
  $rows = sys_get_select($rows,$order,$limit,$fields);
  return $rows;
}
?>