<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

include_once("lib/mail/POP3.php");
include_once("lib/mail/mimeDecode.php");

function _pop3_connect($mfolder) {
  $cid = "pop3_".$mfolder;
  if (empty($GLOBALS["cache"][$cid."_conn"])) {
    if (isset($GLOBALS["credentials"][$cid])) {
      $username = $GLOBALS["credentials"][$cid]["username"];
	  $password = $GLOBALS["credentials"][$cid]["password"];
	  $server = $GLOBALS["credentials"][$cid]["server"];
	  $port = $GLOBALS["credentials"][$cid]["port"];
	  if (!$port) $port = 110;
	  $ssl = $GLOBALS["credentials"][$cid]["ssl"];
	  if ($ssl and !extension_loaded("openssl")) {
	    $GLOBALS["t"]["notification"][] = sprintf("{t}%s is not compiled / loaded into PHP.{/t}","OpenSSL");
	    return false;
	  }
	} else return false;

	$pop3 = new Net_POP3();
	if (PEAR::isError($result = $pop3->connect(($ssl?$ssl."://".$server:$server), $port))) {
	  $GLOBALS["t"]["notification"][] = sprintf("{t}Pop3-conn-error: %s{/t}",$result->getMessage());
	  return false;
	}
	if (PEAR::isError($ret = $pop3->login($username, $password, "USER"))) {
	  $GLOBALS["t"]["notification"][] = sprintf("{t}Pop3-error: %s{/t}",$ret->getMessage());
	  return false;
	}
	$GLOBALS["cache"][$cid."_conn"] = $pop3;
  }
  return $GLOBALS["cache"][$cid."_conn"];
}

function pop3_get_dirs($path, $left, $level, &$tree, $parent, $recursive) {
  $right = $left+1;
  $tree[$left] = array("id"=>$path,"lft"=>$left,"rgt"=>$right,"flevel"=>$level,"ftitle"=>basename($path),"ftype"=>"sys_nosql_pop3");
  return $right+1;
}

function pop3_get_count($path,$where,$vars,$mfolder) {
  $cid = "pop3_count_".md5($mfolder);
  if ($count = sys_cache_get($cid,POP3_LIST_CACHE)) return $count;
  if (!$pop3 = _pop3_connect($mfolder) or !$pop3) return 0;
  if (PEAR::isError($count = $pop3->numMsg())) return 0;
  sys_cache_set($cid,$count);
  return $count;
}

function _pop3_get_datas($mfolder,$cid) {
  if ($datas = sys_cache_get($cid,POP3_LIST_CACHE)) return $datas;
  if (!$pop3 = _pop3_connect($mfolder) or !$pop3) return 0;
  $datas = array();
  $datas[0] = array();
  $datas[1] = $pop3->getListing();
  foreach ($datas[1] as $mail) {
    $datas[0][] = $pop3->getParsedHeaders($mail["msg_id"],"");
  }
  sys_cache_set($cid,$datas);
  return $datas;
}

function _pop3_parse_structure($structure,$id,$uid) {
  $result = array();
  $result["contenttype"] = $structure->ctype_primary."/".$structure->ctype_secondary;
  $result["disposition"] = "inline";
  if (!empty($structure->headers["content-disposition"])) {
    $result["disposition"] = $structure->headers["content-disposition"];
	if ($pos = strpos($result["disposition"],";")) $result["disposition"] = substr($result["disposition"],0,$pos);
  }
  $result["charset"] = "";
  if (isset($structure->ctype_parameters["charset"])) {
    $result["charset"] = strtolower($structure->ctype_parameters["charset"]);
  }
  $result["name"] = str_replace(".","_",$id)."_".$structure->ctype_primary.".".str_replace("plain","txt",$structure->ctype_secondary);
  
  if (!empty($structure->ctype_parameters["name"])) {
	$result["name"] = $structure->ctype_parameters["name"];
  }
  if (!empty($structure->d_parameters["filename"])) {
	$result["name"] = $structure->d_parameters["filename"];
  }

  $result["id"] = $id;
  $result["body"] = "";
  $result["size"] = 0;
  if (isset($structure->body)) {
    $result["size"] = strlen($structure->body);
    if ($result["disposition"]!="attachment" and $result["size"]>0 and $structure->ctype_primary=="text") {
      $result["body"] = &$structure->body;
	}
	if ($structure->ctype_primary=="text" and $structure->ctype_secondary=="calendar") {
	  $result["disposition"]="attachment";
	  if (!strpos($result["name"],".ics")) $result["name"] = "appointment.ics";
	}
    if ($result["disposition"]=="attachment") {
      $local = SIMPLE_CACHE."/pop3/".sha1($uid)."--".urlencode($result["name"]);
      if (!file_exists($local)) {
	    $fp = fopen($local,"wb");
	    fwrite($fp,$structure->body);
	    fclose($fp);
	  }
	}
  }
  $result["header"] = array();
  if (isset($structure->headers)) {
    if (isset($structure->headers["subject"])) $result["header"] = $structure->headers;
  }
  $results = array($result);
  if (isset($structure->parts)) {
    foreach ($structure->parts as $key=>$part) {
	  $results = array_merge($results,_pop3_parse_structure($part,$id.".".($key+1),$uid));
	}
  }
  if (count($results)>1) {
    if ($results[1]["contenttype"]=="text/plain" and $results[2]["contenttype"]=="text/html") {
	  // hide text/html if text/plain is already present
	  $results[2]["contenttype"] = "invalid";
    }
  }
  return $results;
}

function _pop3_drawheader($headers) {
  $content = "Subject: ".modify_decode_subject($headers["subject"])."\n";
  $content .= "Date: ".$headers["date"]."\n";
  $content .= "From: ".modify_decode_subject($headers["from"])."\n";
  $content .= "To: ".modify_decode_subject($headers["to"])."\n";
  return $content;
}

function pop3_get_select($path,$fields,$where,$order,$limit,$vars,$mfolder) {
  $datas = _pop3_get_datas($mfolder,"pop3_".md5($mfolder."/".$path));
  if (!isset($datas[0])) return array();
  $rows = array();
  foreach ($datas[0] as $key=>$data) {
    $msg_id = $datas[1][$key]["msg_id"];
	$msg_uid = $datas[1][$key]["uidl"];
	$structure = array();
	$row = array();
	$row_id = "pop3_fields_".md5(implode("_",$fields).$msg_uid);
	
	if (!$row = sys_cache_get($row_id,POP3_MAIL_CACHE)) {
	  if (!$pop3 = _pop3_connect($mfolder) or !$pop3) return array();
	  foreach ($fields as $field) {
	    switch ($field) {
	      case "id": $row[$field] = $path."/?".md5($msg_uid); break;
	      case "folder": $row[$field] = $path; break;
  		  case "searchcontent": $row[$field] = $data["subject"]." ".$data["from"]; break;
		  case "subject": $row[$field] = isset($data["subject"])?$data["subject"]:""; break;
	 	  case "efrom": $row[$field] = isset($data["from"])?$data["from"]:""; break;
	      case "eto": $row[$field] = isset($data["to"])?$data["to"]:""; break;
  		  case "cc": $row[$field] = isset($data["cc"])?$data["cc"]:""; break;
	  	  case "created":
	      case "lastmodified": $row[$field] = isset($data["date"])?strtotime($data["date"]):"0"; break;
		  case "lastmodifiedby": $row[$field] = ""; break;
	  	  case "dsize":
		  case "size": $row[$field] = $datas[1][$key]["size"]; break;
		  case "headers":
  		    $row[$field] = "";
	  	    foreach ($data as $data_key=>$data_item) $row[$field] .= ucfirst($data_key).": ".(is_array($data_item)?implode("\n  ",$data_item):$data_item)."\n";
		    break;
  		  case "message": 
	  		$row[$field] = "";
			if (empty($structure)) {
			  $decode = new Mail_mimeDecode($pop3->getMsg($msg_id),"\r\n");
			  $structure = _pop3_parse_structure($decode->decode(array("include_bodies"=>true,"decode_bodies"=>true)),1,$msg_uid);
			}
			foreach ($structure as $key2=>$item) {
			  $content = "";
			  switch($item["contenttype"]) {
			    case "multipart/alternative":
				  if (isset($item["header"]["subject"])) $content .= _pop3_drawheader($item["header"]);
				  break;
				default:
				  if ($item["disposition"]!="attachment" and $item["size"]>0 and strpos("@".$item["contenttype"],"text/")) {
				    if ($key2!=0 and isset($item["header"]["subject"])) {
					  $content .= _pop3_drawheader($item["header"]);
					  $content .= "\n----\n\n";
				    }
					$data_body = $item["body"];
					if ($content!="") $content .= "\n";
					if ($item["contenttype"]=="text/html") {
					  $data_body = modify_htmlmessage($data_body);
					} else if ($item["contenttype"]!="text/plain") {
					  $content .= "[".$item["contenttype"]."]\n";
					}
				    $content .= trim($data_body)."\n";
				  }
				  break;
			  }
			  if ($item["charset"]!="utf-8" and (strpos("@".$item["charset"],"iso") or strpos("@".$item["charset"],"ascii"))) $content = utf8_encode($content);
			  if ($row[$field]!="" and $content!="") $row[$field] .= "\n----\n\n";
			  $row[$field] .= $content;
		    }
			break;
		  case "attachments":
			$row[$field] = "";
			if (empty($structure)) {
			  $decode = new Mail_mimeDecode($pop3->getMsg($msg_id),"\r\n");
			  $structure = _pop3_parse_structure($decode->decode(array("include_bodies"=>true,"decode_bodies"=>true)),1,$msg_uid);
			}
			$files = array();
			foreach ($structure as $item) {
			  if ($item["disposition"]=="attachment") {
			    $files[] = SIMPLE_CACHE."/pop3/".sha1($msg_uid)."--".urlencode($item["name"]);
			  }
			}
		    if (count($files)>0) $row[$field] = "|".implode("|",$files)."|";
			break;
		}
	  }
	  sys_cache_set($row_id,$row);
	}
	$row["_msg_id"] = $msg_id;
	$row["seen"] = isset($_SESSION["seen"]["pop3:".$path."/?".md5($msg_uid)])?"checked":"";
	if (sys_get_select_where($row,$where,$vars)) $rows[] = $row;
  }
  $rows = sys_get_select($rows,$order,$limit,$fields);
  return $rows;
}

function pop3_get_delete($path,$where,$vars,$mfolder) {
  if (empty($vars["id"])) return "error";
  if (!$pop3 = _pop3_connect($mfolder) or !$pop3) return "error";

  $datas = $pop3->getListing();
  foreach ($datas as $key=>$data) {
    if ($vars["id"]==$path."/?".md5($data["uidl"])) {
  	  if (PEAR::isError($result = $pop3->deleteMsg($data["msg_id"]))) {
	    $GLOBALS["t"]["notification"][] = sprintf("{t}Pop3-error: %s{/t}",$result->getMessage());
	    return "error";
	  }
	  break;
    }
  }
  $pop3->disconnect();
  unset($GLOBALS["cache"]["pop3_".$mfolder."_conn"]);
  sys_cache_remove("pop3_".md5($mfolder."/".$path));
  sys_cache_remove("pop3_".md5("count_".$mfolder));
  return "";
}

function pop3_get_update($path,$data,$where,$vars,$mfolder) {
  if (empty($vars["id"])) return "error";
  if (isset($data["seen"])) {
	$_SESSION["seen"]["pop3:".$vars["id"]] = "";
	if (count($_SESSION["seen"])>100) $_SESSION["seen"] = array_slice($_SESSION["seen"],-100);
  }
  return "";
}  
?>