<?php
	/**************************************************************************\
	* Simple Groupware 0.241                                                   *
	* http://www.simple-groupware.de                                           *
	* Copyright (C) 2002-2006 by Thomas Bley                                   *
	* ------------------------------------------------------------------------ *
	*  This program is free software; you can redistribute it and/or           *
	*  modify it under the terms of the GNU General Public License Version 2   *
	*  as published by the Free Software Foundation; only version 2            *
	*  of the License, no later version.                                       *
	*                                                                          *
	*  This program is distributed in the hope that it will be useful,         *
	*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
	*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
	*  GNU General Public License for more details.                            *
	*                                                                          *
	*  You should have received a copy of the GNU General Public License       *
	*  Version 2 along with this program; if not, write to the Free Software   *
	*  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                  *
	*  MA  02111-1307, USA.                                                    *
	\**************************************************************************/

if (!defined("MAIN_SCRIPT")) exit;

function _xml_get_data($file) {
  if ($data = sys_cache_get("xml_".sha1($file),XML_CACHE)) return $data;
  if (validate_allowedpath(dirname($file))!="") return array();

  if (is_dir($file)) return array();
  if (!($data = @file_get_contents($file))) {
    sys_message_box("{t}The url doesn't exist.{/t} (".$file.")");
  	return array();
  }
  $data = preg_replace("|<!--.*?-->|msi","",$data);
  $ret = array();
  $parser = xml_parser_create("");
  xml_parser_set_option($parser,XML_OPTION_CASE_FOLDING,0);
  xml_parser_set_option($parser,XML_OPTION_SKIP_WHITE,1);
  xml_parser_set_option($parser, XML_OPTION_TARGET_ENCODING, "utf-8");
  $values = "";
  $tags = array();
  if (!xml_parse_into_struct($parser,$data,$values,$tags)) {
    sys_message_box(sprintf("XML error: %s at line %d",xml_error_string(xml_get_error_code($parser)),xml_get_current_line_number($parser)).$data);
  }
  xml_parser_free($parser);
  $hash_stack = array();
  $a=0;
  foreach ($values as $val) {
    switch ($val['type']) {
      case 'open':
	    array_push($hash_stack, $val['tag']);
        break;
      case 'close':
        array_pop($hash_stack);
        break;
      case 'complete':
        array_push($hash_stack, $val['tag']);
		if (isset($ret[$hash_stack[0]][$hash_stack[1]][$a][$hash_stack[2]])) $a++;
		$ret[$hash_stack[0]][$hash_stack[1]][$a][$hash_stack[2]] = $val["value"];
        array_pop($hash_stack);
        break;
    }
  }
  if (count($ret)>0) {
    $ret = array_values($ret);
	$ret = $ret[0];
	if (count($ret)>0) {
	  sys_cache_set("xml_".sha1($file),$ret);
	  return $ret;
	}
  }
  return array();
}

function xml_get_dirs($path, $left, $level, &$tree, $parent, $recursive) {
  if (validate_allowedpath(dirname($path))!="" or !file_exists($path)) return $left;
  $data = _xml_get_data($path);
  $right = $left+1;
  if ($recursive and count($data)>0 and sys_is_folderstate_open($path,"xml",$parent)) {
	foreach ($data as $folder=>$items) {
	  if (is_array($items) and count($items)>0) {
		$tree[$right] = array("id"=>$path."/".$folder,"lft"=>$right,"rgt"=>$right+1,"flevel"=>$level+1,"ftitle"=>$folder,"ftype"=>"nosql_".basename($path)."_".$folder);
		$right = $right+2;
	  }
	}
  }
  $tree[$left] = array("id"=>$path,"lft"=>$left,"rgt"=>$right,"flevel"=>$level,"ftitle"=>basename($path),"ftype"=>"blank");
  return $right+1;
}

function xml_get_count($path,$where,$vars,$mfolder) {
  $folder = substr($path,strrpos($path,"/")+1);
  $path = substr($path,0,strrpos($path,"/"));
  if (validate_allowedpath(dirname($path))!="" or !file_exists($path)) return 0;
  $data = _xml_get_data($path);
  if (isset($data[$folder]) and is_array($data[$folder])) return count($data[$folder]);
  return 0;
}

function xml_get_select($path,$fields,$where,$order,$limit,$vars,$mfolder) {
  $path = rtrim($path,"/");
  $folder = substr($path,strrpos($path,"/")+1);
  $file_path = substr($path,0,strrpos($path,"/"));
  if (validate_allowedpath(dirname($file_path))!="" or !file_exists($file_path)) return array();
  $datas = _xml_get_data($file_path);
  if (!isset($datas[$folder]) or !is_array($datas[$folder]) or count($datas[$folder])==0) return array();
  $datas = $datas[$folder];
  $rows = array();
  
  $i = 0;
  foreach ($datas as $data) {
	$i++;
    $row = array();
	foreach ($fields as $field) {
	  switch ($field) {
	    case "id": $row[$field] = $path."/?".$i; break;
	    case "folder": $row[$field] = $path; break;
		case "created": $row[$field] = 0; break;
		case "lastmodified": $row[$field] = 0; break;
		case "lastmodifiedby": $row[$field] = ""; break;
		case "searchcontent": $row[$field] = implode(" ",$data); break;
		default:
		  if (!isset($row[$field]) and isset($data[$field])) $row[$field] = $data[$field];
		  break;
	  }
	}
	if (sys_get_select_where($row,$where,$vars)) $rows[] = $row;
  }
  $rows = sys_get_select($rows,$order,$limit,$fields);
  return $rows;
}

?>